"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeDomainService = exports.NodeActions = exports.nodeAccessorID = exports.idleNodeStatuses = exports.unhealthyNodeStatuses = exports.activeNodeStatus = exports.NodeStatus = exports.DEFAULT_FETCH_LIMIT = void 0;
const node_util_1 = require("node:util");
const node_path_1 = __importDefault(require("node:path"));
const node_fs_1 = require("node:fs");
const data_access_layer_1 = require("@jfrog-pipelines/data-access-layer");
const internal_clients_1 = require("@jfrog-pipelines/internal-clients");
const logger_1 = require("@jfrog-pipelines/logger");
const amqp_client_1 = require("@jfrog-pipelines/amqp-client");
const utility_1 = require("@jfrog-pipelines/utility");
const metadata_1 = require("@jfrog-pipelines/metadata");
const errors_1 = require("@jfrog-pipelines/errors");
const configuration_1 = require("@jfrog-pipelines/configuration");
const StatsDomainService_1 = require("../node-stats/StatsDomainService");
const JobDomainService_1 = require("../job/JobDomainService");
const node_console_1 = require("../node-console");
const NodeDao_1 = require("../../driven/db/NodeDao");
const constant_1 = require("../../shared/constant");
const acl_1 = require("../../driven/acl");
const NodePoolService_1 = require("../nodepool/NodePoolService");
const Marshaller_1 = require("./marshaller/Marshaller");
exports.DEFAULT_FETCH_LIMIT = 100;
var NodeStatus;
(function (NodeStatus) {
    NodeStatus[NodeStatus["queued"] = 4000] = "queued";
    NodeStatus[NodeStatus["processing"] = 4001] = "processing";
    NodeStatus[NodeStatus["success"] = 4002] = "success";
    NodeStatus[NodeStatus["failure"] = 4003] = "failure";
    NodeStatus[NodeStatus["waiting"] = 4005] = "waiting";
    NodeStatus[NodeStatus["cancelled"] = 4006] = "cancelled";
    NodeStatus[NodeStatus["unstable"] = 4007] = "unstable";
    NodeStatus[NodeStatus["skipped"] = 4008] = "skipped";
    NodeStatus[NodeStatus["timeout"] = 4009] = "timeout";
    NodeStatus[NodeStatus["stopped"] = 4010] = "stopped";
    NodeStatus[NodeStatus["deleted"] = 4011] = "deleted";
    NodeStatus[NodeStatus["cached"] = 4012] = "cached";
    NodeStatus[NodeStatus["cancelling"] = 4013] = "cancelling";
    NodeStatus[NodeStatus["timingout"] = 4014] = "timingout";
})(NodeStatus = exports.NodeStatus || (exports.NodeStatus = {}));
exports.activeNodeStatus = [
    NodeStatus.success,
    NodeStatus.waiting,
    NodeStatus.stopped,
    NodeStatus.failure,
    NodeStatus.queued,
    NodeStatus.processing,
    NodeStatus.cached
];
exports.unhealthyNodeStatuses = [NodeStatus.failure, NodeStatus.deleted];
exports.idleNodeStatuses = [NodeStatus.waiting, NodeStatus.success, NodeStatus.cached];
exports.nodeAccessorID = '1';
const STOPPED_STATE_CODE = 80;
var NodeActions;
(function (NodeActions) {
    NodeActions[NodeActions["CREATE"] = 10] = "CREATE";
    NodeActions[NodeActions["CACHE"] = 20] = "CACHE";
    NodeActions[NodeActions["START"] = 30] = "START";
    NodeActions[NodeActions["DELETE"] = 40] = "DELETE";
})(NodeActions = exports.NodeActions || (exports.NodeActions = {}));
class NodeDomainService {
    constructor(certificateInitializer) {
        this.logger = logger_1.LoggerFactory.getLogger('NodeService');
        this.nodeDao = new NodeDao_1.NodeDao();
        this.userService = new acl_1.UserRepository();
        this.accessClient = internal_clients_1.AccessClientFactory.getClient();
        this.nodeConsoleAdapter = new node_console_1.NodeConsoleLedgerService();
        this.nodeStatsService = new StatsDomainService_1.StatsDomainService();
        this.cloudMarshaller = new Marshaller_1.Marshaller(this.nodeConsoleAdapter);
        this.nodePoolService = new NodePoolService_1.NodePoolService();
        this.buildPlaneMessageService = new JobDomainService_1.JobDomainService();
        this.userRepository = new acl_1.UserRepository();
        this.properties = configuration_1.ConfigFactory.config;
        this.certificateInitializer = certificateInitializer;
    }
    async createNode(attribute) {
        try {
            let node = await this.nodeDao.create(attribute);
            const nodeToken = await this.getNodeAccessToken(node);
            this.logger.trace(`Successfully created nodeToken for ${node.friendlyName}`);
            node = await this.updateNodeBootStrapToken(node, nodeToken);
            await this.createNodeInstance(node);
            await this.sendWWWSignal(node);
            return node;
        }
        catch (error) {
            this.logger.warn(`Failed to create node due to error : ${(0, node_util_1.inspect)(error)}`);
            throw error;
        }
    }
    async createBetaNode(nodeCreateAttributes, runtime) {
        const node = await this.nodeDao.create(nodeCreateAttributes);
        this.logger.debug(`Node with node pool id ${nodeCreateAttributes.nodePoolId} created successfully with name ${node.friendlyName}`);
        const nodeToken = await this.getNodeAccessToken(node);
        this.logger.trace(`Successfully created nodeToken for ${node.friendlyName}`);
        await this.updateNodeBootStrapToken(node, nodeToken);
        await this.createNodeInstance(node);
        this.logger.trace(`Successfully provisioned node for ${node.friendlyName}`);
        await this.sendWWWSignal(node);
        return node;
    }
    async getRunningNodes(poolID) {
        const queryBuilder = new data_access_layer_1.SequelizeQueryBuilder({})
            .where(`nodePoolId`, poolID)
            .where('statusCode', exports.activeNodeStatus)
            .where('stopRequested', 'false');
        return this.nodeDao.findAll(queryBuilder.build());
    }
    async getWaitingNodes(poolID) {
        const queryBuilder = new data_access_layer_1.SequelizeQueryBuilder({})
            .where(`nodePoolId`, poolID)
            .where('statusCode', NodeStatus.waiting)
            .where('stopRequested', 'false');
        return this.nodeDao.findAll(queryBuilder.build());
    }
    async getAllNodes(poolID) {
        const queryBuilder = new data_access_layer_1.SequelizeQueryBuilder({}).where(`nodePoolId`, poolID);
        return this.nodeDao.findAll(queryBuilder.build());
    }
    async getOrphanNodes() {
        const queryBuilder = new data_access_layer_1.SequelizeQueryBuilder({}).where(`nodePoolId`, null);
        return this.nodeDao.findAll(queryBuilder.build());
    }
    async applyNodeDeletedState(node) {
        try {
            await this.stopNode(node);
            await this.resetBuildPlaneMessage(node);
            if (!(0, utility_1.isEmpty)(node.instanceId)) {
                await this.cloudMarshaller.deleteInstance(node);
                await this.nodeDao.updateByInstance(node, {
                    instanceInfo: {},
                    instanceId: null,
                    providerMetadataPropertyBag: {}
                });
            }
            return await this.removeNodeAndReference(node.id);
        }
        catch (error) {
            if (error instanceof errors_1.InstanceNotFoundError) {
                this.logger.info(`Removing instance reference since instance is already deleted on cloud for node ${node.id} and nodepool ${node.nodePoolId}`);
                return this.removeNodeAndReference(node.id);
            }
            await this.applyNodeState(node, NodeStatus.failure);
            this.logger.warn(`Failed to apply delete state to node ${node.id} of nodePoolId ${node.nodePoolId} due to ${(0, node_util_1.inspect)(error)}`);
            throw error;
        }
    }
    async isNodeStopped(node) {
        const projectIntegration = await this.userRepository.getProjectIntegrationById(node.projectIntegrationId);
        if (projectIntegration.masterIntegrationName !== 'amazonKeys') {
            return true;
        }
        try {
            const instanceStatus = await this.cloudMarshaller.getNodeStatus(node);
            return instanceStatus === STOPPED_STATE_CODE;
        }
        catch (error) {
            if (error instanceof errors_1.TooManyRequestError) {
                return false;
            }
            throw error;
        }
    }
    async applyNodeToCachedState(node) {
        let appliedNode = null;
        try {
            appliedNode = await this.applyNodeState(node, NodeStatus.stopped);
            await this.resetBuildPlaneMessage(node);
            await this.cloudMarshaller.suspendInstance(node);
        }
        catch (error) {
            if (error instanceof errors_1.StateTransitionError || error instanceof errors_1.TooManyRequestError) {
                return appliedNode;
            }
            throw error;
        }
        return appliedNode;
    }
    async applyNodeWaitingState(node) {
        try {
            await this.cloudMarshaller.resumeInstance(node);
            return await this.applyNodeState(node, NodeStatus.waiting);
        }
        catch (error) {
            if (error instanceof errors_1.CloudUnauthorizedError) {
                this.logger.warn(`Marking node ${node.id} of nodepool ${node.nodePoolId} as failure since instance is unauthorized to be started`);
                await this.applyNodeState(node, NodeStatus.failure);
            }
            if (!this.isNodeFailed(error)) {
                return node;
            }
            throw error;
        }
    }
    async getNodeInitScript(nodeId, nodePoolIds) {
        const unifiedUrl = await this.userService.getUnifiedUiAndWwwUrl();
        const rtUiUrl = unifiedUrl.rtUiUrl;
        const node = await this.getNode(nodeId, nodePoolIds);
        if (!node) {
            throw new errors_1.NotFoundError(`Node node found for id: ${nodeId} or you don't have sufficient permission`);
        }
        const nodePool = await this.nodePoolService.getNodePool(node.nodePoolId);
        if (!nodePool) {
            throw new errors_1.NotFoundError(`NodePool not found for nodePoolId: ${node.nodePoolId}`);
        }
        const scriptFileExtension = metadata_1.NodeMetadata.getScriptFileExtension(nodePool.architecture, nodePool.operatingSystem);
        const buildPlaneArtifactExtension = metadata_1.NodeMetadata.getBuildPlaneArtifactExtension(nodePool.architecture, nodePool.operatingSystem);
        this.validateSupport(nodePool, buildPlaneArtifactExtension, scriptFileExtension);
        return this.generateInitScript(node, nodePool, rtUiUrl, buildPlaneArtifactExtension, scriptFileExtension);
    }
    async applyNodeState(node, status) {
        try {
            this.logger.trace(`Updating the node ${node.id}:${node.friendlyName} with status: ${status}`);
            const updateResponse = await this.nodeDao.updateByInstance(node, {
                statusCode: status,
                statusLastUpdatedAt: new Date()
            });
            return updateResponse;
        }
        catch (error) {
            this.logger.warn(`Failed to applyNodeState to ${status} for node id ${node.id} due to err ${(0, node_util_1.inspect)(error)} `);
        }
        return node;
    }
    getNode(nodeId, nodePoolIds) {
        const query = new data_access_layer_1.SequelizeQueryBuilder({}).where('id', nodeId);
        if (nodePoolIds) {
            query.where('nodePoolId', nodePoolIds);
        }
        return this.nodeDao.findOne(query.build());
    }
    generateInitScript(node, nodePool, rtUiUrl, buildPlaneArtifactExtension, scriptFileExtension) {
        let scriptPath = `./marshaller/templates/userDataStaticBash.tpl`;
        if (nodePool.operatingSystem === 'WindowsServer_2019') {
            scriptPath = `./marshaller/templates/userDataStaticWin.tpl`;
        }
        const filePath = node_path_1.default.join(__dirname, scriptPath);
        const fileContent = (0, node_fs_1.readFileSync)(filePath).toString();
        const scriptData = this.prepareInitScriptData(node, nodePool, rtUiUrl, buildPlaneArtifactExtension);
        const templateData = (0, utility_1.renderTemplateWithoutPath)(fileContent, scriptData);
        const fileName = `node_init_${node.id}.${scriptFileExtension}`;
        return {
            scriptData: templateData,
            scriptName: fileName
        };
    }
    prepareInitScriptData(node, nodePool, rtUiUrl, buildPlaneArtifactExtension) {
        const externalApiUrl = this.properties.externalApiUrl;
        const parameters = {
            nodeId: node.id,
            projectId: node.projectId,
            buildPlaneVersion: node.intendedBuildPlaneVersion,
            apiUrl: externalApiUrl,
            baseUrlUI: rtUiUrl,
            runMode: this.properties.runMode,
            architecture: nodePool.architecture,
            operatingSystem: nodePool.operatingSystem,
            buildPlaneArtifactDownloadUrl: `${externalApiUrl}/passthrough/artifacts/buildPlane-${nodePool.architecture}-${nodePool.operatingSystem}.${buildPlaneArtifactExtension}`,
            configureNode: false,
            installPrereqs: true,
            isSwapEnabled: false,
            noVerifySsl: this.properties.noVerifySsl,
            customCertsData: this.getCustomCertData(),
            customCertsEnabled: this.properties.customCertsEnabled,
            httpProxy: process.env.http_proxy ?? '',
            httpsProxy: process.env.https_proxy ?? '',
            noProxy: process.env.no_proxy ?? '',
            token: node.systemPropertyBag?.token,
            nodePollerIntervalMS: this.properties.nodePollerIntervalMS,
            rootUserInit: !nodePool.initAsNonRoot,
            insecureDockerRegistries: this.getInsecureDockerRegistryParams(node)
        };
        this.setStaticNodeProperties(parameters, node);
        return parameters;
    }
    getCustomCertData() {
        const certInitializer = this.certificateInitializer;
        return certInitializer?.customCertsData;
    }
    getInsecureDockerRegistryParams(node) {
        return JSON.stringify(node.initPropertyBag?.insecureDockerRegistries ?? []);
    }
    setStaticNodeProperties(parameters, node) {
        if (!node.isOnDemand) {
            parameters.installPrereqs = true;
            this.setSwapProperties(parameters, node);
            this.setProxyProperties(parameters);
        }
    }
    setSwapProperties(parameters, node) {
        if (node.isSwapEnabled) {
            parameters.configureNode = true;
            parameters.isSwapEnabled = true;
        }
    }
    setProxyProperties(parameters) {
        if ((0, utility_1.isEmpty)(parameters.httpProxy) || (0, utility_1.isEmpty)(parameters.httpsProxy) || (0, utility_1.isEmpty)(parameters.noProxy)) {
            parameters.configureNode = true;
        }
    }
    validateSupport(nodePool, buildPlaneArtifactExtension, scriptFileExtension) {
        if (!metadata_1.NodeMetadata.isSupportedOperatingSystem(nodePool.architecture, nodePool.operatingSystem)) {
            throw new errors_1.BadRequestError(`${nodePool.operatingSystem} has reached end of support. ` +
                `Please change the OS to another supported version. ${constant_1.TROUBLESHOOT_MSG}`);
        }
        if ((0, utility_1.isEmpty)(buildPlaneArtifactExtension) || (0, utility_1.isEmpty)(scriptFileExtension)) {
            throw new errors_1.BadRequestError(`node init files not found for architecture: ${nodePool.architecture} & operating system: ${nodePool.operatingSystem}`);
        }
    }
    async createNodeInstance(node) {
        try {
            const instanceCreatedResponse = await this.cloudMarshaller.createInstance(node);
            await this.nodeDao.updateByInstance(node, {
                instanceInfo: instanceCreatedResponse.instanceInfo,
                instanceId: instanceCreatedResponse.instanceId,
                provisionedAt: new Date(),
                statusCode: metadata_1.WAITING,
                statusLastUpdatedAt: new Date(),
                providerMetadataPropertyBag: {
                    ...node.providerMetadataPropertyBag
                }
            });
            this.logger.trace(`Successfully provision node for ${node.friendlyName}`);
        }
        catch (error) {
            if (error instanceof errors_1.TooManyRequestError) {
                this.logger.warn(`Marking instance as failed for node id ${node.id} in nodepool ${node.nodePoolId} due to error TooManyRequestError`);
            }
            else {
                this.logger.warn(`Failed to create instance for node id ${node.id} in nodepool ${node.nodePoolId} due to error ${(0, node_util_1.inspect)(error)}`);
            }
            await this.nodeDao.updateByInstance(node, { statusCode: metadata_1.FAILED });
        }
    }
    async getNodeAccessToken(node) {
        return this.accessClient.getNodeAccessToken(node);
    }
    async updateNodeBootStrapToken(node, token, txn) {
        return this.nodeDao.updateByInstance(node, { systemPropertyBag: { token: token } }, { transaction: txn });
    }
    async sendWWWSignal(node) {
        const publisher = new amqp_client_1.AmqpPublisher(amqp_client_1.AmqpFactory.getConnectionManager(), this.properties.amqpExchange, amqp_client_1.publisherOptions, this.logger);
        const signalPayload = {
            eventType: `node.created`,
            rooms: `v3/projects/${node.projectId}`,
            payload: { id: node.id }
        };
        return publisher.publishMessage(`www.signals`, signalPayload);
    }
    async resetBuildPlaneMessage(node) {
        try {
            await this.buildPlaneMessageService.resetBuildPlane(node);
        }
        catch (error) {
            this.logger.warn(`Failed to reset build plane message for ${node.id} ${(0, node_util_1.inspect)(error)}`);
        }
    }
    async stopNode(node) {
        try {
            this.logger.info(`Stopping and revoking access for node ${node.id} `);
            await Promise.all([
                this.accessClient.revokeToken(node.systemPropertyBag.token),
                this.nodeDao.updateByInstance(node, {
                    statusCode: metadata_1.STOPPED
                })
            ]);
        }
        catch (error) {
            this.logger.warn(`Failed to stop node ${node.id} due to ${error.message}`);
        }
    }
    async removeNodeAndReference(nodeId) {
        await this.nodeStatsService.deleteByNodeId(nodeId);
        await this.nodeConsoleAdapter.deleteByNodeId(nodeId);
        const queryBuilder = new data_access_layer_1.SequelizeQueryBuilder({});
        queryBuilder.where('id', nodeId);
        return this.nodeDao.delete(queryBuilder.build());
    }
    isNodeFailed(error) {
        if (error instanceof errors_1.TooManyRequestError || error instanceof errors_1.StateTransitionError) {
            return false;
        }
        return true;
    }
}
exports.NodeDomainService = NodeDomainService;
//# sourceMappingURL=NodeDomainService.js.map