"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const amqp_client_1 = require("@jfrog-pipelines/amqp-client");
const internal_clients_1 = require("@jfrog-pipelines/internal-clients");
const logger_1 = require("@jfrog-pipelines/logger");
const configuration_1 = require("@jfrog-pipelines/configuration");
const NodeCollector_1 = require("../NodeCollector");
const NodeDomainService_1 = require("../NodeDomainService");
const NodePoolService_1 = require("../../nodepool/NodePoolService");
const Marshaller_1 = require("../marshaller/Marshaller");
const StatsDomainService_1 = require("../../node-stats/StatsDomainService");
const NodeConsoleLedgerService_1 = require("../../node-console/NodeConsoleLedgerService");
const acl_1 = require("../../../driven/acl");
jest.mock('@jfrog-pipelines/amqp-client');
jest.mock('@jfrog/access-nodejs-client');
jest.mock('@jfrog-pipelines/redis', () => {
    const actualRedis = jest.requireActual('@jfrog-pipelines/redis');
    return {
        ...actualRedis,
        RedisClientFactory: {
            client: {}
        }
    };
});
jest.mock('../../../driven/acl/userService');
jest.mock('@jfrog-pipelines/logger', () => {
    const actualTracker = jest.requireActual('@jfrog-pipelines/logger');
    return {
        ...actualTracker,
        LoggerFactory: {
            getLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            }),
            configure: jest.fn(),
            logEvent: jest.fn(),
            getEventLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            })
        }
    };
});
const publishMessageMock = jest.spyOn(amqp_client_1.AmqpPublisher.prototype, 'publishMessage').mockResolvedValueOnce({});
jest.spyOn(internal_clients_1.AccessClientFactory, 'getClient').mockReturnValue({
    getNodeAccessToken: jest.fn().mockReturnValue('token')
});
jest.spyOn(configuration_1.ConfigFactory, 'config', 'get').mockImplementation(() => {
    return { rtRouterJoinKey: 'abcd1234' };
});
describe('Test NodeCollector #deProvision', () => {
    const MS = 1000;
    const nodeCollector = new NodeCollector_1.NodeCollector();
    const nodePool = { id: 1, name: 'testpool', projectId: 1, isCacheEnabled: true, isOnDemand: true };
    const getNodePoolMock = jest.spyOn(NodePoolService_1.NodePoolService.prototype, 'getNodePool').mockResolvedValueOnce(nodePool);
    const oneMin = 60 * MS;
    const sixteenMins = 16 * oneMin;
    const sixteenMinsBefore = new Date(Date.now() - sixteenMins);
    afterEach(() => {
        publishMessageMock.mockClear();
        getNodePoolMock.mockClear();
    });
    const STATUS_UPDATED_TIMEOUT = 15 * 60000;
    const PROVISIONED_DATE_TIME = 15 * 24 * 60 * 60000;
    const nodes = [
        { friendlyName: 'test1', projectId: 1, nodePoolId: 1, statusCode: NodeDomainService_1.NodeStatus.cached },
        {
            friendlyName: 'test2',
            projectId: 1,
            nodePoolId: 1,
            statusCode: NodeDomainService_1.NodeStatus.success,
            statusLastUpdatedAt: new Date(Date.now() - STATUS_UPDATED_TIMEOUT),
            provisionedAt: new Date(Date.now() - PROVISIONED_DATE_TIME)
        }
    ];
    it('should delete success nodes when statusLastUpdatedAt > nodeIdleInterval', async () => {
        const oneMin = 60 * MS;
        const oneMinuteBefore = new Date(Date.now() - oneMin);
        const nodes = [
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeDomainService_1.NodeStatus.success,
                statusLastUpdatedAt: oneMinuteBefore
            },
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeDomainService_1.NodeStatus.success,
                statusLastUpdatedAt: new Date()
            }
        ];
        const getRunningNodesMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const np = {
            ...nodePool,
            isCacheEnabled: false,
            nodeIdleIntervalInMins: 1
        };
        const eventLoggerSpy = jest.spyOn(logger_1.LoggerFactory, 'logEvent');
        const applyNodeDeletedStateMock = jest
            .spyOn(NodeDomainService_1.NodeDomainService.prototype, 'applyNodeDeletedState')
            .mockResolvedValueOnce(1);
        jest.spyOn(Marshaller_1.Marshaller.prototype, 'deleteInstance').mockImplementation(jest.fn());
        jest.spyOn(StatsDomainService_1.StatsDomainService.prototype, 'deleteByNodeId').mockImplementation(jest.fn());
        jest.spyOn(NodeConsoleLedgerService_1.NodeConsoleLedgerService.prototype, 'deleteByNodeId').mockImplementation(jest.fn());
        const deProvisionResponse = await nodeCollector.deProvision(np, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeDeletedStateMock).toHaveBeenCalledTimes(1);
        expect(eventLoggerSpy).toHaveBeenCalledTimes(1);
        getRunningNodesMock.mockClear();
        applyNodeDeletedStateMock.mockClear();
    });
    it('should cache success nodes when statusLastUpdatedAt > nodeIdleInterval when isCacheEnabled is true', async () => {
        const oneMin = 60 * MS;
        const oneMinuteBefore = new Date(Date.now() - oneMin);
        const nodes = [
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                id: 1,
                statusCode: NodeDomainService_1.NodeStatus.success,
                statusLastUpdatedAt: oneMinuteBefore
            },
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                id: 2,
                statusCode: NodeDomainService_1.NodeStatus.success,
                statusLastUpdatedAt: new Date()
            }
        ];
        const getRunningNodesMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const np = {
            ...nodePool,
            isCacheEnabled: true,
            nodeIdleIntervalInMins: 1
        };
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeDomainService_1.NodeStatus.stopped };
        const applyNodeToCachedStateSpy = jest
            .spyOn(NodeDomainService_1.NodeDomainService.prototype, 'applyNodeToCachedState')
            .mockResolvedValueOnce(node);
        const deProvisionResponse = await nodeCollector.deProvision(np, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeToCachedStateSpy).toHaveBeenCalledTimes(1);
        getRunningNodesMock.mockClear();
        applyNodeToCachedStateSpy.mockClear();
    });
    it('should  not change deleted nodes based on nodeStopSetting', async () => {
        const getRunningNodesMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'getRunningNodes').mockResolvedValueOnce(nodes);
        nodePool.cacheSettingsPropertyBag = { nodeStopSetting: '' };
        const applyNodeDeletedStateMock = jest
            .spyOn(NodeDomainService_1.NodeDomainService.prototype, 'applyNodeDeletedState')
            .mockResolvedValueOnce(1);
        const deProvisionResponse = await nodeCollector.deProvision(nodePool, 0);
        await expect(deProvisionResponse).toBe(undefined);
        getRunningNodesMock.mockClear();
        applyNodeDeletedStateMock.mockClear();
    });
    it('should delete cached node based on nodeStopIntervalDays', async () => {
        const oneDay = 24 * 60 * 60 * MS;
        const twoDays = 2 * oneDay;
        const oneDayBefore = new Date(Date.now() - oneDay);
        const twoDayBefore = new Date(Date.now() - twoDays);
        const nodes = [
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeDomainService_1.NodeStatus.cached,
                provisionedAt: oneDayBefore,
                statusLastUpdatedAt: oneDayBefore
            },
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeDomainService_1.NodeStatus.cached,
                provisionedAt: twoDayBefore,
                statusLastUpdatedAt: oneDayBefore
            }
        ];
        const getRunningNodesMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const np = {
            ...nodePool,
            isCacheEnabled: true,
            nodeIdleIntervalInMins: 1,
            cacheSettingsPropertyBag: { nodeStopIntervalDays: 1, nodeStopSetting: 'nodeStopIntervalDays' }
        };
        const applyNodeDeletedStateMock = jest
            .spyOn(NodeDomainService_1.NodeDomainService.prototype, 'applyNodeDeletedState')
            .mockResolvedValueOnce(1);
        const deProvisionResponse = await nodeCollector.deProvision(np, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeDeletedStateMock).toHaveBeenCalledTimes(2);
        getRunningNodesMock.mockClear();
        applyNodeDeletedStateMock.mockClear();
    });
    it('should not delete cached node which is provisioned today', async () => {
        const oneMin = 60 * MS;
        const sixteenMins = 16 * oneMin;
        const sixteenMinsBefore = new Date(Date.now() - sixteenMins);
        const nodes = [
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeDomainService_1.NodeStatus.cached,
                provisionedAt: sixteenMinsBefore,
                statusLastUpdatedAt: sixteenMinsBefore,
                instanceId: 'instanceId1'
            },
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeDomainService_1.NodeStatus.cached,
                provisionedAt: sixteenMinsBefore,
                statusLastUpdatedAt: sixteenMinsBefore,
                instanceId: 'instanceId2'
            }
        ];
        const getRunningNodesMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const np = {
            ...nodePool,
            isCacheEnabled: true,
            nodeIdleIntervalInMins: 1,
            cacheSettingsPropertyBag: { nodeStopIntervalDays: 1, nodeStopSetting: 'nodeStopIntervalDays' }
        };
        const applyNodeDeletedStateMock = jest
            .spyOn(NodeDomainService_1.NodeDomainService.prototype, 'applyNodeDeletedState')
            .mockResolvedValueOnce(1);
        const deProvisionResponse = await nodeCollector.deProvision(np, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeDeletedStateMock).toHaveBeenCalledTimes(0);
        getRunningNodesMock.mockClear();
        applyNodeDeletedStateMock.mockClear();
    });
    it('should delete cached node which is configured based on nodeStopDayOfWeek', async () => {
        jest.useFakeTimers();
        const fakeDate = new Date();
        fakeDate.setUTCFullYear(2023, 1, 5);
        fakeDate.setHours(0, 0, 0);
        jest.setSystemTime(fakeDate);
        const oneMin = 60 * MS;
        const sixteenMins = 16 * oneMin;
        const sixteenMinsBefore = new Date(Date.now() - sixteenMins);
        const nodes = [
            {
                id: 1,
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeDomainService_1.NodeStatus.cached,
                provisionedAt: sixteenMinsBefore,
                statusLastUpdatedAt: sixteenMinsBefore
            },
            {
                id: 2,
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeDomainService_1.NodeStatus.cached,
                provisionedAt: sixteenMinsBefore,
                statusLastUpdatedAt: sixteenMinsBefore
            }
        ];
        const getRunningNodesMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const np = {
            ...nodePool,
            isCacheEnabled: true,
            nodeIdleIntervalInMins: 1,
            cacheSettingsPropertyBag: { nodeStopSetting: 'nodeStopDayOfWeek', nodeStopDayOfWeek: 'sunday' }
        };
        const applyNodeDeletedStateMock = jest
            .spyOn(NodeDomainService_1.NodeDomainService.prototype, 'applyNodeDeletedState')
            .mockResolvedValueOnce(1);
        const deProvisionResponse = await nodeCollector.deProvision(np, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeDeletedStateMock).toHaveBeenCalledTimes(2);
        getRunningNodesMock.mockClear();
        applyNodeDeletedStateMock.mockClear();
        jest.useRealTimers();
    });
    const sunday = new Date();
    sunday.setUTCFullYear(2023, 9, 1);
    sunday.setHours(0, 0, 0);
    const monday = new Date();
    monday.setUTCFullYear(2023, 9, 2);
    monday.setHours(0, 0, 0);
    const previousSaturday = new Date();
    previousSaturday.setUTCFullYear(2023, 8, 29);
    previousSaturday.setHours(0, 0, 0);
    const nextSaturday = new Date();
    nextSaturday.setUTCFullYear(2023, 9, 7);
    nextSaturday.setHours(0, 0, 0);
    const nextSunday = new Date();
    nextSunday.setUTCFullYear(2023, 9, 8);
    nextSunday.setHours(0, 0, 0);
    const table = [
        {
            systemDate: sunday,
            provisionedAt: monday,
            statusLastUpdatedAt: monday,
            cacheSettingsPropertyBag: { nodeStopSetting: 'nodeStopDayOfWeek', nodeStopDayOfWeek: 'sunday' },
            expectedNodesToBeDeleted: 0
        },
        {
            systemDate: sunday,
            provisionedAt: sunday,
            statusLastUpdatedAt: sunday,
            cacheSettingsPropertyBag: { nodeStopSetting: 'nodeStopDayOfWeek', nodeStopDayOfWeek: 'sunday' },
            expectedNodesToBeDeleted: 0
        },
        {
            systemDate: sunday,
            provisionedAt: previousSaturday,
            statusLastUpdatedAt: previousSaturday,
            cacheSettingsPropertyBag: { nodeStopSetting: 'nodeStopDayOfWeek', nodeStopDayOfWeek: 'sunday' },
            expectedNodesToBeDeleted: 1
        },
        {
            systemDate: nextSunday,
            provisionedAt: nextSaturday,
            statusLastUpdatedAt: nextSaturday,
            cacheSettingsPropertyBag: { nodeStopSetting: 'nodeStopDayOfWeek', nodeStopDayOfWeek: 'sunday' },
            expectedNodesToBeDeleted: 1
        },
        {
            systemDate: nextSunday,
            provisionedAt: sunday,
            statusLastUpdatedAt: nextSaturday,
            cacheSettingsPropertyBag: { nodeStopSetting: 'nodeStopDayOfWeek', nodeStopDayOfWeek: 'sunday' },
            expectedNodesToBeDeleted: 1
        },
        {
            systemDate: monday,
            provisionedAt: sunday,
            statusLastUpdatedAt: sunday,
            cacheSettingsPropertyBag: { nodeStopIntervalDays: 1, nodeStopSetting: 'nodeStopIntervalDays' },
            expectedNodesToBeDeleted: 1
        },
        {
            systemDate: nextSunday,
            provisionedAt: sunday,
            statusLastUpdatedAt: monday,
            cacheSettingsPropertyBag: { nodeStopIntervalDays: 15, nodeStopSetting: 'nodeStopIntervalDays' },
            expectedNodesToBeDeleted: 0
        }
    ];
    it.each(table)('given systemDate: $systemDate, nodeProvisionedA: $provisionedAt, statusLastUpdatedAt:$statusLastUpdatedAt, cacheSettingsPropertyBag: $cacheSettingsPropertyBag then expectedNodesToBeDeleted: $expectedNodesToBeDeleted', async ({ systemDate, provisionedAt, statusLastUpdatedAt, cacheSettingsPropertyBag, expectedNodesToBeDeleted }) => {
        jest.useFakeTimers();
        jest.setSystemTime(systemDate);
        const nodes = [
            {
                id: 1,
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeDomainService_1.NodeStatus.cached,
                provisionedAt: provisionedAt,
                statusLastUpdatedAt: statusLastUpdatedAt
            }
        ];
        const getRunningNodesMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const np = {
            ...nodePool,
            isCacheEnabled: true,
            nodeIdleIntervalInMins: 1,
            cacheSettingsPropertyBag: cacheSettingsPropertyBag
        };
        const applyNodeDeletedStateMock = jest
            .spyOn(NodeDomainService_1.NodeDomainService.prototype, 'applyNodeDeletedState')
            .mockResolvedValueOnce(1);
        const deProvisionResponse = await nodeCollector.deProvision(np, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeDeletedStateMock).toHaveBeenCalledTimes(expectedNodesToBeDeleted);
        getRunningNodesMock.mockClear();
        applyNodeDeletedStateMock.mockClear();
        jest.useRealTimers();
    });
    it('should  not delete node based on nodeProvisionedAt', async () => {
        const getRunningNodesMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'getRunningNodes').mockResolvedValueOnce(nodes);
        nodePool.cacheSettingsPropertyBag = { nodeStopIntervalDays: 20, nodeStopSetting: 'nodeStopIntervalDays' };
        const applyNodeDeletedStateMock = jest
            .spyOn(NodeDomainService_1.NodeDomainService.prototype, 'applyNodeDeletedState')
            .mockResolvedValueOnce(1);
        const deProvisionResponse = await nodeCollector.deProvision(nodePool, 0);
        await expect(deProvisionResponse).toBe(undefined);
        getRunningNodesMock.mockClear();
        applyNodeDeletedStateMock.mockClear();
    });
    it('should not deprovision node which are waiting', async () => {
        const nodes = [
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeDomainService_1.NodeStatus.waiting,
                provisionedAt: sixteenMinsBefore,
                statusLastUpdatedAt: new Date()
            },
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeDomainService_1.NodeStatus.waiting,
                provisionedAt: sixteenMinsBefore,
                statusLastUpdatedAt: new Date()
            }
        ];
        const getRunningNodesMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const np = { ...nodePool, nodeIdleIntervalInMins: 2, isCacheEnabled: true };
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeDomainService_1.NodeStatus.waiting };
        const applyNodeDeletedStateMock = jest
            .spyOn(NodeDomainService_1.NodeDomainService.prototype, 'applyNodeDeletedState')
            .mockResolvedValueOnce(1);
        const applyNodeToCachedStateMock = jest
            .spyOn(NodeDomainService_1.NodeDomainService.prototype, 'applyNodeToCachedState')
            .mockResolvedValueOnce(node);
        const deProvisionResponse = await nodeCollector.deProvision(np, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeToCachedStateMock).toHaveBeenCalledTimes(0);
        expect(applyNodeDeletedStateMock).toHaveBeenCalledTimes(0);
        getRunningNodesMock.mockClear();
        applyNodeDeletedStateMock.mockClear();
    });
    it('should  delete node based on nodeProvisionedAt', async () => {
        const nodes = [
            { friendlyName: 'test1', projectId: 1, nodePoolId: 1, statusCode: NodeDomainService_1.NodeStatus.cached },
            {
                friendlyName: 'test2',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeDomainService_1.NodeStatus.success,
                statusLastUpdatedAt: new Date(),
                provisionedAt: new Date()
            }
        ];
        const getRunningNodesMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'getRunningNodes').mockResolvedValueOnce(nodes);
        nodePool.cacheSettingsPropertyBag = { nodeStopIntervalDays: 20, nodeStopSetting: 'nodeStopIntervalDays' };
        const applyNodeDeletedStateMock = jest
            .spyOn(NodeDomainService_1.NodeDomainService.prototype, 'applyNodeDeletedState')
            .mockResolvedValueOnce(1);
        const deProvisionResponse = await nodeCollector.deProvision(nodePool, 0);
        await expect(deProvisionResponse).toBe(undefined);
        getRunningNodesMock.mockClear();
        applyNodeDeletedStateMock.mockClear();
    });
    it('should delete failure nodes', async () => {
        const nodes = [{ friendlyName: 'test1', projectId: 1, nodePoolId: 1, statusCode: NodeDomainService_1.NodeStatus.failure }];
        const getRunningNodesMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const applyNodeDeletedStateMock = jest
            .spyOn(NodeDomainService_1.NodeDomainService.prototype, 'applyNodeDeletedState')
            .mockResolvedValueOnce(1);
        const deProvisionResponse = await nodeCollector.deProvision(nodePool, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeDeletedStateMock).toHaveBeenCalledTimes(1);
        getRunningNodesMock.mockClear();
        applyNodeDeletedStateMock.mockClear();
    });
    it('should delete waiting nodes which have crossed cuttoffTime', async () => {
        const np = {
            ...nodePool,
            isCacheEnabled: true,
            nodeIdleIntervalInMins: 1
        };
        const nodes = [
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeDomainService_1.NodeStatus.waiting,
                statusLastUpdatedAt: sixteenMinsBefore
            }
        ];
        const getRunningNodesMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const applyNodeDeletedStateMock = jest
            .spyOn(NodeDomainService_1.NodeDomainService.prototype, 'applyNodeDeletedState')
            .mockResolvedValueOnce(1);
        const deProvisionResponse = await nodeCollector.deProvision(np, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeDeletedStateMock).toHaveBeenCalledTimes(1);
        getRunningNodesMock.mockClear();
        applyNodeDeletedStateMock.mockClear();
    });
    it('should not delete having stopRequested = true', async () => {
        const np = {
            ...nodePool,
            isCacheEnabled: true,
            nodeIdleIntervalInMins: 1
        };
        const nodes = [
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeDomainService_1.NodeStatus.waiting,
                stopRequested: true
            },
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeDomainService_1.NodeStatus.success,
                stopRequested: true
            },
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeDomainService_1.NodeStatus.failure,
                stopRequested: true
            },
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeDomainService_1.NodeStatus.stopped,
                stopRequested: true
            },
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeDomainService_1.NodeStatus.cached,
                stopRequested: true
            }
        ];
        const getRunningNodesMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const applyNodeDeletedStateMock = jest
            .spyOn(NodeDomainService_1.NodeDomainService.prototype, 'applyNodeDeletedState')
            .mockResolvedValueOnce(1);
        jest.spyOn(acl_1.UserRepository.prototype, 'getProjectIntegrationById').mockResolvedValue({
            masterIntegrationName: 'amazonKeys'
        });
        jest.spyOn(Marshaller_1.Marshaller.prototype, 'getNodeStatus').mockResolvedValue(80);
        const deProvisionResponse = await nodeCollector.deProvision(np, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeDeletedStateMock).toHaveBeenCalledTimes(5);
        getRunningNodesMock.mockClear();
        applyNodeDeletedStateMock.mockClear();
    });
    it('should not delete stopped nodes', async () => {
        const nodes = [{ friendlyName: 'test1', projectId: 1, nodePoolId: 1, statusCode: NodeDomainService_1.NodeStatus.stopped }];
        const getRunningNodesMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const applyNodeDeletedStateMock = jest
            .spyOn(NodeDomainService_1.NodeDomainService.prototype, 'applyNodeDeletedState')
            .mockResolvedValueOnce(1);
        const deProvisionResponse = await nodeCollector.deProvision(nodePool, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeDeletedStateMock).toHaveBeenCalledTimes(0);
        getRunningNodesMock.mockClear();
        applyNodeDeletedStateMock.mockClear();
    });
    it('should delete cached nodes', async () => {
        const nodes = [
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeDomainService_1.NodeStatus.cached,
                statusLastUpdatedAt: sixteenMinsBefore,
                provisionedAt: sixteenMinsBefore
            }
        ];
        const getRunningNodesMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const np = {
            ...nodePool,
            isCacheEnabled: false,
            nodeIdleIntervalInMins: 1
        };
        const applyNodeDeletedStateMock = jest
            .spyOn(NodeDomainService_1.NodeDomainService.prototype, 'applyNodeDeletedState')
            .mockResolvedValueOnce(1);
        const deProvisionResponse = await nodeCollector.deProvision(np, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeDeletedStateMock).toHaveBeenCalledTimes(1);
        getRunningNodesMock.mockClear();
        applyNodeDeletedStateMock.mockClear();
    });
    it('should not delete processing nodes', async () => {
        const nodes = [
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeDomainService_1.NodeStatus.processing,
                provisionedAt: sixteenMinsBefore,
                statusLastUpdatedAt: sixteenMinsBefore
            },
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeDomainService_1.NodeStatus.processing,
                provisionedAt: sixteenMinsBefore,
                statusLastUpdatedAt: sixteenMinsBefore
            }
        ];
        const getRunningNodesMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const applyNodeDeletedStateMock = jest
            .spyOn(NodeDomainService_1.NodeDomainService.prototype, 'applyNodeDeletedState')
            .mockResolvedValueOnce(1);
        const deProvisionResponse = await nodeCollector.deProvision(nodePool, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeDeletedStateMock).toHaveBeenCalledTimes(0);
        getRunningNodesMock.mockClear();
        applyNodeDeletedStateMock.mockClear();
    });
    it('should delete deleted nodes', async () => {
        const nodes = [{ friendlyName: 'test1', projectId: 1, nodePoolId: 1, statusCode: NodeDomainService_1.NodeStatus.deleted }];
        const getRunningNodesMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const np = {
            ...nodePool,
            isCacheEnabled: false,
            nodeIdleIntervalInMins: 1
        };
        const applyNodeDeletedStateMock = jest
            .spyOn(NodeDomainService_1.NodeDomainService.prototype, 'applyNodeDeletedState')
            .mockResolvedValueOnce(1);
        const deProvisionResponse = await nodeCollector.deProvision(np, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeDeletedStateMock).toHaveBeenCalledTimes(1);
        getRunningNodesMock.mockClear();
        applyNodeDeletedStateMock.mockClear();
    });
    it('should delete failure and deleted nodes', async () => {
        const nodes = [
            { friendlyName: 'test1', projectId: 1, nodePoolId: 1, statusCode: NodeDomainService_1.NodeStatus.stopped },
            { friendlyName: 'test1', projectId: 1, nodePoolId: 1, statusCode: NodeDomainService_1.NodeStatus.failure },
            { friendlyName: 'test1', projectId: 1, nodePoolId: 1, statusCode: NodeDomainService_1.NodeStatus.deleted }
        ];
        const getRunningNodesMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const applyNodeDeletedStateMock = jest
            .spyOn(NodeDomainService_1.NodeDomainService.prototype, 'applyNodeDeletedState')
            .mockResolvedValueOnce(1);
        const deProvisionResponse = await nodeCollector.deProvision(nodePool, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeDeletedStateMock).toHaveBeenCalledTimes(2);
        getRunningNodesMock.mockClear();
        applyNodeDeletedStateMock.mockClear();
    });
    it('should  collect orphan nodes', async () => {
        const nodes = [
            { friendlyName: 'test1', projectId: 1, nodePoolId: null, statusCode: NodeDomainService_1.NodeStatus.waiting },
            { friendlyName: 'test2', projectId: 1, nodePoolId: null, statusCode: NodeDomainService_1.NodeStatus.success },
            { friendlyName: 'test3', projectId: 1, nodePoolId: null, statusCode: NodeDomainService_1.NodeStatus.cached },
            { friendlyName: 'test4', projectId: 1, nodePoolId: null, statusCode: NodeDomainService_1.NodeStatus.processing }
        ];
        const getRunningNodesMock = jest.spyOn(NodeDomainService_1.NodeDomainService.prototype, 'getOrphanNodes').mockResolvedValueOnce(nodes);
        const applyNodeDeletedStateMock = jest
            .spyOn(NodeDomainService_1.NodeDomainService.prototype, 'applyNodeDeletedState')
            .mockImplementation(jest.fn());
        const orphanNodesCollected = await nodeCollector.collectJunkNodes();
        await expect(orphanNodesCollected).toBe(4);
        getRunningNodesMock.mockClear();
        applyNodeDeletedStateMock.mockClear();
    });
});
//# sourceMappingURL=NodeCollector.test.js.map