"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const amqp_client_1 = require("@jfrog-pipelines/amqp-client");
const internal_clients_1 = require("@jfrog-pipelines/internal-clients");
const configuration_1 = require("@jfrog-pipelines/configuration");
const microservice_1 = require("@jfrog-pipelines/microservice");
const NodeDao_1 = require("../../../driven/db/NodeDao");
const StatsDomainService_1 = require("../../node-stats/StatsDomainService");
const NodeConsoleLedgerService_1 = require("../../node-console/NodeConsoleLedgerService");
const Marshaller_1 = require("../marshaller/Marshaller");
jest.mock('../../../driven/acl/userService', () => {
    return {
        UserRepository: jest.fn().mockReturnValue({
            getProjectIntegrationById: jest.fn().mockResolvedValue({
                formJSONValues: [{ label: 'key', value: 'value' }],
                masterIntegrationName: 'amazonKeys'
            })
        })
    };
});
const NodeDomainService_1 = require("../NodeDomainService");
jest.mock('@jfrog-pipelines/redis', () => {
    const actualRedis = jest.requireActual('@jfrog-pipelines/redis');
    return {
        ...actualRedis,
        RedisClientFactory: {
            client: {}
        }
    };
});
jest.mock('./../marshaller/Marshaller');
jest.mock('@jfrog-pipelines/data-access-layer', () => {
    const actualDataAccessLayer = jest.requireActual('@jfrog-pipelines/data-access-layer');
    return {
        ...actualDataAccessLayer,
        SequelizeTransactionManager: jest.fn().mockReturnValue({
            getTransaction: jest.fn().mockResolvedValue({ commit: jest.fn(), rollback: jest.fn() })
        })
    };
});
jest.mock('@jfrog-pipelines/configuration', () => {
    const actualTracker = jest.requireActual('@jfrog-pipelines/configuration');
    return {
        ...actualTracker,
        config: {
            apiToken: 'sdfgsd',
            instanceType: 't3a.medium',
            getCustom: jest.fn(),
            customCertsEnabled: false,
            isSaas: true
        },
        configure: jest.fn()
    };
});
jest.mock('@jfrog/access-nodejs-client');
jest.mock('@jfrog-pipelines/amqp-client');
jest.mock('@jfrog-pipelines/logger', () => {
    const actualTracker = jest.requireActual('@jfrog-pipelines/logger');
    return {
        ...actualTracker,
        LoggerFactory: {
            getLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            }),
            configure: jest.fn()
        }
    };
});
const publishMessageMock = jest.spyOn(amqp_client_1.AmqpPublisher.prototype, 'publishMessage').mockResolvedValueOnce({});
jest.spyOn(configuration_1.ConfigFactory, 'config', 'get').mockImplementation(() => {
    return { rtRouterJoinKey: 'abcd1234' };
});
jest.spyOn(internal_clients_1.AccessClientFactory, 'getClient').mockReturnValue({
    getNodeAccessToken: jest.fn().mockReturnValue('token'),
    revokeToken: jest.fn().mockImplementation(jest.fn())
});
jest
    .spyOn(NodeDao_1.NodeDao.prototype, 'updateByInstance')
    .mockImplementation(async (model, update) => {
    return { ...model, ...update };
});
const statusCodeList = [4002, 4005, 4010, 4003, 4000, 4001, 4012];
describe('Test NodeDomainService #createNode', () => {
    afterEach(() => {
        publishMessageMock.mockClear();
        jest.clearAllMocks();
    });
    it('should be able to create node based on node create attributes', async () => {
        const nodeCreateAttributes = {
            friendlyName: 'test',
            projectId: 1,
            nodePoolId: 1,
            systemPropertyBag: {
                token: 'token'
            }
        };
        const dao = new NodeDao_1.NodeDao();
        const expectedValue = {
            friendlyName: 'test',
            projectId: 1,
            nodePoolId: 1,
            systemPropertyBag: { token: 'token' }
        };
        const createMock = jest.spyOn(NodeDao_1.NodeDao.prototype, 'create').mockResolvedValueOnce(expectedValue);
        jest
            .spyOn(Marshaller_1.Marshaller.prototype, 'createInstance')
            .mockResolvedValueOnce({ instanceId: 'test', instanceInfo: {}, meta: {} });
        const service = new NodeDomainService_1.NodeDomainService(microservice_1.applicationContext);
        const gotResponse = await service.createNode(nodeCreateAttributes);
        expect(dao.create).toHaveBeenCalledWith(nodeCreateAttributes);
        expect(gotResponse).toStrictEqual(expectedValue);
        createMock.mockClear();
    });
});
describe('Test NodeDomainService #getRunningNodes', () => {
    afterEach(() => {
        publishMessageMock.mockClear();
        jest.clearAllMocks();
    });
    it('should return nodes with active status', async () => {
        const dao = new NodeDao_1.NodeDao();
        const expectedResponse = [
            { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeDomainService_1.NodeStatus.success }
        ];
        const findAllMock = jest.spyOn(NodeDao_1.NodeDao.prototype, 'findAll').mockResolvedValueOnce(expectedResponse);
        const service = new NodeDomainService_1.NodeDomainService(microservice_1.applicationContext);
        const gotResponse = await service.getRunningNodes(1);
        expect(dao.findAll).toHaveBeenCalledWith({
            where: { nodePoolId: 1, statusCode: statusCodeList, stopRequested: 'false' }
        });
        expect(gotResponse).toBe(expectedResponse);
        findAllMock.mockClear();
    });
    it('should return null if no active nodes found', async () => {
        const dao = new NodeDao_1.NodeDao();
        const findAllMock = jest.spyOn(NodeDao_1.NodeDao.prototype, 'findAll').mockImplementationOnce(() => Promise.resolve([]));
        const service = new NodeDomainService_1.NodeDomainService(microservice_1.applicationContext);
        const gotResponse = await service.getRunningNodes(1);
        expect(dao.findAll).toHaveBeenCalledWith({
            where: { nodePoolId: 1, statusCode: statusCodeList, stopRequested: 'false' }
        });
        expect(gotResponse).toStrictEqual([]);
        findAllMock.mockClear();
    });
});
describe('Test NodeDomainService #getWaitingNodes', () => {
    afterEach(() => {
        publishMessageMock.mockClear();
        jest.clearAllMocks();
    });
    it('should return nodes with active status', async () => {
        const dao = new NodeDao_1.NodeDao();
        const expectedResponse = [
            { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeDomainService_1.NodeStatus.waiting }
        ];
        const findAllMock = jest.spyOn(NodeDao_1.NodeDao.prototype, 'findAll').mockResolvedValueOnce(expectedResponse);
        const service = new NodeDomainService_1.NodeDomainService(microservice_1.applicationContext);
        const gotResponse = await service.getWaitingNodes(1);
        expect(dao.findAll).toHaveBeenCalledWith({ where: { nodePoolId: 1, statusCode: 4005, stopRequested: 'false' } });
        expect(gotResponse).toBe(expectedResponse);
        findAllMock.mockClear();
    });
    it('should return null if no active nodes found', async () => {
        const dao = new NodeDao_1.NodeDao();
        const findAllMock = jest.spyOn(NodeDao_1.NodeDao.prototype, 'findAll').mockImplementationOnce(() => Promise.resolve([]));
        const service = new NodeDomainService_1.NodeDomainService(microservice_1.applicationContext);
        const gotResponse = await service.getWaitingNodes(1);
        expect(dao.findAll).toHaveBeenCalledWith({ where: { nodePoolId: 1, statusCode: 4005, stopRequested: 'false' } });
        expect(gotResponse).toStrictEqual([]);
        findAllMock.mockClear();
    });
});
describe('Test NodeDomainService #applyNodeDeletedState', () => {
    afterEach(() => {
        publishMessageMock.mockClear();
        jest.clearAllMocks();
    });
    it('should change the node status to stopped', async () => {
        const node = {
            id: 1,
            friendlyName: 'test',
            projectId: 1,
            nodePoolId: 1,
            statusCode: NodeDomainService_1.NodeStatus.stopped,
            instanceId: 'testinstanceid',
            systemPropertyBag: {
                token: 'token'
            }
        };
        jest.spyOn(NodeDao_1.NodeDao.prototype, 'delete').mockResolvedValue(1);
        jest.spyOn(Marshaller_1.Marshaller.prototype, 'deleteInstance').mockImplementation(jest.fn());
        jest.spyOn(StatsDomainService_1.StatsDomainService.prototype, 'deleteByNodeId').mockImplementation(jest.fn());
        jest.spyOn(NodeConsoleLedgerService_1.NodeConsoleLedgerService.prototype, 'deleteByNodeId').mockImplementation(jest.fn());
        const service = new NodeDomainService_1.NodeDomainService(microservice_1.applicationContext);
        await service.applyNodeDeletedState(node);
        expect(NodeDao_1.NodeDao.prototype.delete).toHaveBeenCalledTimes(1);
        expect(Marshaller_1.Marshaller.prototype.deleteInstance).toHaveBeenCalledTimes(1);
        expect(StatsDomainService_1.StatsDomainService.prototype.deleteByNodeId).toHaveBeenCalledTimes(1);
        expect(NodeConsoleLedgerService_1.NodeConsoleLedgerService.prototype.deleteByNodeId).toHaveBeenCalledTimes(1);
    });
});
describe('Test NodeDomainService #applyNodeToCachedState', () => {
    afterEach(() => {
        publishMessageMock.mockClear();
        jest.clearAllMocks();
    });
    it('should change node status to cached', async () => {
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeDomainService_1.NodeStatus.cached };
        const dao = new NodeDao_1.NodeDao();
        const service = new NodeDomainService_1.NodeDomainService(microservice_1.applicationContext);
        const expectedResponse = await service.applyNodeToCachedState(node);
        expect(dao.updateByInstance).toHaveBeenCalled();
        expect(expectedResponse.statusCode).toBe(NodeDomainService_1.NodeStatus.stopped);
    });
});
describe('Test NodeDomainService #applyNodeWaitingState', () => {
    afterEach(() => {
        publishMessageMock.mockClear();
        jest.clearAllMocks();
    });
    it('should change node status from cached to waiting', async () => {
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeDomainService_1.NodeStatus.waiting };
        const dao = new NodeDao_1.NodeDao();
        const service = new NodeDomainService_1.NodeDomainService(microservice_1.applicationContext);
        const expectedResponse = await service.applyNodeWaitingState(node);
        expect(dao.updateByInstance).toHaveBeenCalled();
        expect(expectedResponse.statusCode).toBe(NodeDomainService_1.NodeStatus.waiting);
    });
});
//# sourceMappingURL=NodeService.test.js.map