"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeCreator = void 0;
const node_util_1 = require("node:util");
const internal_clients_1 = require("@jfrog-pipelines/internal-clients");
const logger_1 = require("@jfrog-pipelines/logger");
const amqp_client_1 = require("@jfrog-pipelines/amqp-client");
const configuration_1 = require("@jfrog-pipelines/configuration");
const metadata_1 = require("@jfrog-pipelines/metadata");
const driven_1 = require("../../../driven");
const WWW_QUEUE_NAME = 'www.signals';
const NODE_CREATE_EVENT = 'node.created';
const SIGNAL_ROOM_PATTERN = 'v3/projects/%s';
class NodeCreator {
    constructor(nodeFields) {
        this.nodeFields = nodeFields;
        this.logger = logger_1.LoggerFactory.getLogger('NodeCreator');
        this.nodeDao = new driven_1.NodeDao();
        this.accessClient = internal_clients_1.AccessClientFactory.getClient();
        this.properties = configuration_1.ConfigFactory.config;
    }
    async process() {
        this.validateNodeConfig(this.nodeFields);
        const node = await this.nodeDao.create(this.nodeFields);
        const nodeToken = await this.getNodeAccessToken(node);
        this.logger.trace(`Successfully created nodeToken for ${node.friendlyName}`);
        return this.updateNodeBootStrapToken(node, nodeToken);
    }
    async sendSignal() {
        const { projectId, id } = this.nodeFields;
        const publisher = new amqp_client_1.AmqpPublisher(amqp_client_1.AmqpFactory.getConnectionManager(), this.properties.amqpExchange, amqp_client_1.publisherOptions, this.logger);
        const signalPayload = {
            eventType: NODE_CREATE_EVENT,
            rooms: (0, node_util_1.format)(SIGNAL_ROOM_PATTERN, projectId),
            payload: { id }
        };
        return publisher.publishMessage(WWW_QUEUE_NAME, signalPayload);
    }
    async updateNodeCreation(node, instanceCreatedResponse) {
        await this.nodeDao.updateByInstance(node, {
            instanceInfo: instanceCreatedResponse.instanceInfo,
            instanceId: instanceCreatedResponse.instanceId,
            provisionedAt: new Date(),
            statusCode: metadata_1.WAITING,
            statusLastUpdatedAt: new Date(),
            providerMetadataPropertyBag: {
                ...node.providerMetadataPropertyBag
            }
        });
    }
    async getNodeAccessToken(node) {
        return this.accessClient.getNodeAccessToken(node);
    }
    async updateNodeBootStrapToken(node, token, txn) {
        return this.nodeDao.updateByInstance(node, { systemPropertyBag: { token: token } }, { transaction: txn });
    }
    validateNodeConfig(node) {
        const region = node.providerMetadataPropertyBag.region;
        const imageId = node.providerMetadataPropertyBag.imageId;
        if (region === '' || imageId == '') {
            throw new Error(`No image id or region specified`);
        }
    }
}
exports.NodeCreator = NodeCreator;
//# sourceMappingURL=NodeCreator.js.map