"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AwsDriver = void 0;
const node_util_1 = require("node:util");
const node_path_1 = __importDefault(require("node:path"));
const logger_1 = require("@jfrog-pipelines/logger");
const cloud_1 = require("@jfrog-pipelines/cloud");
const utility_1 = require("@jfrog-pipelines/utility");
const metadata_1 = require("@jfrog-pipelines/metadata");
const configuration_1 = require("@jfrog-pipelines/configuration");
const microservice_1 = require("@jfrog-pipelines/microservice");
const errors_1 = require("@jfrog-pipelines/errors");
const NodeCreator_1 = require("../../NodeCreator");
const driven_1 = require("../../../../../driven");
const AwsRetryPolicy_1 = require("./AwsRetryPolicy");
const AwsErrorHandler_1 = require("./AwsErrorHandler");
const TEMPLATE_PATH = 'templates/userData.tpl';
const MAC_OPERATING_SYSTEMS = new Set(['MacOS_13.5']);
class AwsDriver {
    constructor(awsClient, instanceVariables, nodePool, node) {
        this.awsClient = awsClient;
        this.instanceVariables = instanceVariables;
        this.nodePool = nodePool;
        this.logger = logger_1.LoggerFactory.getLogger('AwsDriver');
        this.retryManager = new AwsRetryPolicy_1.AwsRetryPolicy(this.logger);
        this.node = node ?? null;
        this.userRepository = new driven_1.UserRepository();
        this.systemServiceRepository = new driven_1.SystemServiceRepository();
        this.properties = configuration_1.ConfigFactory.config;
    }
    static create({ accessKeyId, secretAccessKey, region }, instanceVariables, node, nodePool) {
        const awsClient = cloud_1.AwsClient.create({ accessKeyId, secretAccessKey, region });
        return new AwsDriver(awsClient, instanceVariables, nodePool, node);
    }
    async create() {
        const nodeCreationAttributes = await this.nodePool.getNodeCreationAttributes();
        const nodeCreator = new NodeCreator_1.NodeCreator(nodeCreationAttributes);
        const nodeEntity = await nodeCreator.process();
        const awsInstanceResponse = await this.getNodeCreateRequest(nodeEntity);
        const createdResponse = {
            instanceInfo: awsInstanceResponse,
            instanceId: awsInstanceResponse.instanceId,
            meta: {
                instanceId: awsInstanceResponse.InstanceId,
                imageId: awsInstanceResponse.ImageId,
                instanceType: awsInstanceResponse.InstanceType,
                launchTime: awsInstanceResponse.LaunchTime,
                privateIpAddress: awsInstanceResponse.PrivateIpAddress,
                subnetId: awsInstanceResponse.SubnetId,
                vpcId: awsInstanceResponse.VpcId
            }
        };
        await nodeCreator.updateNodeCreation(nodeEntity, createdResponse);
        await nodeCreator.sendSignal();
        return createdResponse;
    }
    async terminate() {
        if (!this.node) {
            return;
        }
        const [instanceId, clientToken] = [this.node.getInstanceId().getValue(), this.node.getClientToken().getValue()];
        if (instanceId) {
            return this.terminateInstance(instanceId);
        }
        if (clientToken) {
            return this.terminateMacHost(clientToken);
        }
        throw new Error('Neither InstanceId nor HostId is provided');
    }
    async suspend() {
        try {
            if (!this.node) {
                return;
            }
            const instanceId = this.node.getInstanceId().getValue();
            await this.retryManager.execute(this.awsClient.stopInstance.bind(this.awsClient), [instanceId]);
        }
        catch (error) {
            AwsErrorHandler_1.AwsErrorHandler.handle(error);
        }
    }
    async start() {
        try {
            if (!this.node) {
                return;
            }
            const instanceId = this.node.getInstanceId().getValue();
            await this.retryManager.execute(this.awsClient.startInstance.bind(this.awsClient), [instanceId]);
        }
        catch (error) {
            AwsErrorHandler_1.AwsErrorHandler.handle(error);
        }
    }
    async getCurrentStatus() {
        if (!this.node) {
            return 0;
        }
        const instanceId = this.node.getInstanceId().getValue();
        const instance = await this.retryManager.execute(this.awsClient.describeInstanceStatus.bind(this.awsClient), [instanceId]);
        if (instance.InstanceStatuses?.length) {
            return instance.InstanceStatuses[0].InstanceState?.Code;
        }
    }
    async terminateInstance(instanceId) {
        try {
            await this.retryManager.execute(this.awsClient.terminateInstance.bind(this.awsClient), [[instanceId]]);
        }
        catch (error) {
            AwsErrorHandler_1.AwsErrorHandler.handle(error);
        }
    }
    async getNodeBootTemplate(node) {
        const templatePath = node_path_1.default.join(__dirname, TEMPLATE_PATH);
        try {
            const userData = await this.getNodeUserData(node);
            return (0, utility_1.renderTemplate)(templatePath, userData);
        }
        catch (error) {
            this.logger.error(`Failed to load template due to error: ${(0, node_util_1.inspect)(error)}`);
            throw error;
        }
    }
    getCustomCertData() {
        const certInitialzer = microservice_1.applicationContext.get(microservice_1.CERTIFICATE_INITIALIZER);
        return certInitialzer?.customCertsData;
    }
    async getNodeUserData(node) {
        const runMode = this.properties.runMode;
        const nodePollerIntervalMS = this.properties.nodePollerIntervalMS;
        const isCustomCertsEnabled = this.properties.customCertsEnabled;
        const customCertsData = this.getCustomCertData();
        const noVerifySsl = this.properties.noVerifySsl;
        const urlConfig = await this.userRepository.getUnifiedUiAndWwwUrl();
        let externalApiUrl = await this.systemServiceRepository.getExternalURL();
        if (externalApiUrl.trim() == '') {
            externalApiUrl = configuration_1.ConfigFactory.config.externalApiUrl;
        }
        const buildPlaneArtifactExtension = metadata_1.NodeMetadata.getBuildPlaneArtifactExtension(this.nodePool.getArchitecture().getValue(), this.nodePool.getOperatingSystem().getValue());
        const downloadImageUrl = `${urlConfig.rtUiUrl}/pipelines/webapp/buildPlane-${this.nodePool
            .getArchitecture()
            .getValue()}-${this.nodePool.getOperatingSystem().getValue()}.${buildPlaneArtifactExtension}`;
        const token = node.systemPropertyBag.token;
        let insecureDockerRegistries = JSON.stringify([]);
        if (this.nodePool.getInitPropertyBag().insecureDockerRegistries) {
            insecureDockerRegistries = JSON.stringify(this.nodePool.getInitPropertyBag().insecureDockerRegistries);
        }
        return {
            nodeId: node.id,
            projectId: Number.parseInt(this.nodePool.getProjectId().getValue(), 10),
            buildPlaneVersion: node.intendedBuildPlaneVersion,
            apiExternalUrl: externalApiUrl,
            baseUrlUI: urlConfig.rtUiUrl,
            runMode: runMode,
            architecture: this.nodePool.getArchitecture().getValue(),
            operatingSystem: this.nodePool.getOperatingSystem().getValue(),
            buildPlaneArtifactDownloadUrl: downloadImageUrl,
            configureNode: false,
            installPrereqs: false,
            isSwapEnabled: false,
            token: token,
            nodePollerIntervalMS: nodePollerIntervalMS,
            noVerifySsl: noVerifySsl,
            customCertsEnabled: isCustomCertsEnabled,
            customCertsData: customCertsData && JSON.stringify(customCertsData),
            rootUserInit: !this.nodePool.getInitAsNonRoot(),
            diskSizeinGB: this.nodePool.getDiskSizeinGB().getValue(),
            insecureDockerRegistries: insecureDockerRegistries
        };
    }
    async terminateMacHost(clientToken) {
        await this.awsClient.terminateMacHost(clientToken);
    }
    async getNodeCreateRequest(nodeEntity) {
        const project = await this.nodePool.getProject();
        const nodeUserData = await this.getNodeBootTemplate(nodeEntity);
        const encodedNodeUserData = Buffer.from(nodeUserData).toString('base64');
        const providerMetadata = this.nodePool.getProviderMetadataPropertyBag();
        const tags = this.transformTags(providerMetadata.customTags);
        const operatingSystem = this.nodePool.getOperatingSystem().getValue();
        if (MAC_OPERATING_SYSTEMS.has(operatingSystem)) {
            const hostResponse = await this.createMacNode(nodeEntity);
            if (hostResponse.Hosts?.length) {
                return hostResponse.Hosts[0];
            }
            throw new Error(`No created host found for node id ${nodeEntity.id} with reponse : ${(0, node_util_1.inspect)(hostResponse)}`);
        }
        const createdResponse = await this.createEC2Node(nodeEntity, encodedNodeUserData, project, tags);
        if (createdResponse.Instances?.length) {
            return createdResponse.Instances[0];
        }
        throw new Error(`No created instances found for node id ${nodeEntity.id} with reponse : ${(0, node_util_1.inspect)(createdResponse)}`);
    }
    async createMacNode(nodeEntity) {
        const clientToken = '';
        if (clientToken.trim().length === 0) {
            throw new errors_1.BadRequestError('No client token found');
        }
        const availabilityZone = await this.awsClient.getAvailabilityZone(this.nodePool.getProviderMetadataPropertyBag().subnetId);
        const tagSpecifications = this.getHostTagSpecifications(nodeEntity);
        const allocatedHost = await this.awsClient.allocateDedicatedHosts(availabilityZone, this.nodePool.getProviderMetadataPropertyBag().instanceSize, clientToken, tagSpecifications);
        if (!allocatedHost) {
            throw new Error('unable to create host');
        }
        return this.awsClient.describeHost(clientToken);
    }
    async createEC2Node(nodeEntity, encodedNodeUserData, project, tags) {
        const tagSpecifications = this.getInstanceTagSpecifications(nodeEntity, project, tags);
        const blockDeviceMappings = this.getBlockDeviceMappings();
        return this.createEC2(nodeEntity, tagSpecifications, blockDeviceMappings, encodedNodeUserData);
    }
    async createEC2(nodeEntity, tagSpecifications, blockDeviceMappings, instanceData) {
        try {
            const runParameters = {
                imageId: nodeEntity.providerMetadataPropertyBag.imageId,
                subnetId: this.nodePool.getProviderMetadataPropertyBag().subnetId,
                instanceName: nodeEntity.friendlyName,
                instanceSize: this.nodePool.getProviderMetadataPropertyBag().instanceSize,
                securityGroupId: this.nodePool.getProviderMetadataPropertyBag().securityGroupId,
                keyName: this.nodePool.getProviderMetadataPropertyBag().keyPairName,
                userData: instanceData,
                tagSpecifications: tagSpecifications,
                hostId: undefined,
                clientToken: '',
                blockDeviceMappings: blockDeviceMappings
            };
            const createInstanceResponse = await this.retryManager.execute(this.awsClient.createInstance.bind(this.awsClient), [runParameters]);
            return createInstanceResponse;
        }
        catch (error) {
            AwsErrorHandler_1.AwsErrorHandler.handle(error);
        }
        throw new Error('Failure createEC2');
    }
    getBlockDeviceMappings() {
        const blockDeviceMappings = [];
        if (this.nodePool.getDiskSizeinGB().getValue()) {
            blockDeviceMappings.push({
                DeviceName: '/dev/sda1',
                Ebs: {
                    VolumeSize: this.nodePool.getDiskSizeinGB().getValue(),
                    VolumeType: 'gp2'
                }
            });
        }
        return blockDeviceMappings;
    }
    getInstanceTagSpecifications(nodeEntity, project, tags) {
        let instanceTags = [
            {
                Key: 'Project Id',
                Value: `${nodeEntity.projectId}`
            },
            {
                Key: 'Project',
                Value: project.name
            },
            {
                Key: 'Node Pool Id',
                Value: this.nodePool.getId().getValue()
            },
            {
                Key: 'NodePool',
                Value: this.nodePool.getName()
            },
            {
                Key: 'Node Name',
                Value: nodeEntity.friendlyName
            },
            {
                Key: 'Node Id',
                Value: `${nodeEntity.id}`
            }
        ];
        let volumeTags = [];
        if (this.instanceVariables.isSaaS()) {
            const saasTags = [
                {
                    Key: 'Customer Name',
                    Value: this.instanceVariables.getSaasCustomerName()
                },
                {
                    Key: 'SaaS Environment',
                    Value: this.instanceVariables.getSaasEnvironment()
                },
                {
                    Key: 'Customer',
                    Value: this.instanceVariables.getSaasCustomerName()
                }
            ];
            instanceTags = [...instanceTags, ...saasTags];
            volumeTags = saasTags;
        }
        if (!(0, utility_1.isEmpty)(tags) && Array.isArray(tags)) {
            instanceTags = [...instanceTags, ...tags];
        }
        const tagSpecifications = [
            {
                ResourceType: 'instance',
                Tags: instanceTags
            }
        ];
        if (this.instanceVariables.isSaaS()) {
            tagSpecifications.push({
                ResourceType: 'volume',
                Tags: volumeTags
            });
        }
        return tagSpecifications;
    }
    getHostTagSpecifications(nodeEntity) {
        let hostTags = [
            {
                Key: 'Project Id',
                Value: `${nodeEntity.projectId}`
            },
            {
                Key: 'Node Pool Id',
                Value: this.nodePool.getId().getValue()
            },
            {
                Key: 'Node Name',
                Value: nodeEntity.friendlyName
            },
            {
                Key: 'Node Id',
                Value: `${nodeEntity.id}`
            }
        ];
        if (this.instanceVariables.isSaaS()) {
            const saasTags = [
                {
                    Key: 'Customer Name',
                    Value: this.instanceVariables.getSaasCustomerName()
                },
                {
                    Key: 'SaaS Environment',
                    Value: this.instanceVariables.getSaasEnvironment()
                },
                {
                    Key: 'Customer',
                    Value: this.instanceVariables.getSaasCustomerName()
                }
            ];
            hostTags = [...hostTags, ...saasTags];
        }
        const tagSpecifications = [
            {
                ResourceType: 'dedicated-host',
                Tags: hostTags
            }
        ];
        return tagSpecifications;
    }
    transformTags(tags) {
        const awsTags = [];
        if (!tags) {
            return awsTags;
        }
        for (const tag of tags) {
            if (tag.Key && tag.Value) {
                awsTags.push({
                    Key: tag.Key,
                    Value: tag.Value
                });
            }
        }
        return awsTags;
    }
}
exports.AwsDriver = AwsDriver;
//# sourceMappingURL=AwsDriver.js.map