"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AwsRetryPolicy = void 0;
const cockatiel_1 = require("cockatiel");
const errors_1 = require("@jfrog-pipelines/errors");
const AwsErrorCodes_1 = require("./AwsErrorCodes");
const AwsErrorHandler_1 = require("./AwsErrorHandler");
const EXPONENTIAL_RETRY_INITIAL_DELAY = 1000;
const EXPONENTIAL_RETRY_MAX_ATTEMPT = 5;
const DEFAULT_ERROR_MESSAGE = 'AWS_ERROR';
class AwsRetryPolicy {
    constructor(logger) {
        this.logger = logger;
    }
    async execute(retryableFunction, inputs) {
        const policy = (0, cockatiel_1.handleWhen)((error) => error instanceof errors_1.TooManyRequestError);
        const policyOptions = {
            maxAttempts: EXPONENTIAL_RETRY_MAX_ATTEMPT,
            backoff: new cockatiel_1.ExponentialBackoff({ initialDelay: EXPONENTIAL_RETRY_INITIAL_DELAY })
        };
        const retryPolicy = (0, cockatiel_1.retry)(policy, policyOptions);
        try {
            let numberOfRetriesAttempted = 0;
            const result = await retryPolicy.execute(async () => {
                try {
                    return await retryableFunction(...inputs);
                }
                catch (error) {
                    const awsError = error;
                    if (awsError.error?.code === AwsErrorCodes_1.REQUEST_LIMIT_EXCEEDED) {
                        this.logger.warn(`Request Failed with Too Many Requests, retrying again ${numberOfRetriesAttempted}`);
                        numberOfRetriesAttempted = numberOfRetriesAttempted + 1;
                        const errorMessage = awsError.error.message ?? DEFAULT_ERROR_MESSAGE;
                        throw new errors_1.TooManyRequestError(errorMessage);
                    }
                    throw error;
                }
            });
            return result;
        }
        catch (error) {
            const awsErrorMessage = error.error?.message ?? '';
            this.logger.warn(`Request failed with ${awsErrorMessage}, aborting`);
            AwsErrorHandler_1.AwsErrorHandler.handle(error);
        }
        throw new Error('Failed to process request');
    }
}
exports.AwsRetryPolicy = AwsRetryPolicy;
//# sourceMappingURL=AwsRetryPolicy.js.map