"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CloudServiceClient = void 0;
const node_util_1 = require("node:util");
const cloud_1 = require("@jfrog-pipelines/cloud");
const configuration_1 = require("@jfrog-pipelines/configuration");
const microservice_1 = require("@jfrog-pipelines/microservice");
const userService_1 = require("../../../driven/acl/userService");
const SystemServiceRepository_1 = require("../../../driven/acl/systemService/SystemServiceRepository");
const AwsRequestTransformer_1 = require("./AwsRequestTransformer");
const GcloudRequestTransformer_1 = require("./GcloudRequestTransformer");
const K8sRequestTransformer_1 = require("./K8sRequestTransformer");
const AzureRequestTransformer_1 = require("./AzureRequestTransformer");
const providerType = {
    amazonKeys: 'aws',
    gcloudKey: 'gce',
    azureKeys: 'azure',
    kubernetesConfig: 'k8s'
};
class CloudServiceClient {
    constructor() {
        this.userRepository = new userService_1.UserRepository();
        this.systemServiceRepository = new SystemServiceRepository_1.SystemServiceRepository();
    }
    async getNodeProvider(node, region, zone) {
        try {
            const projectIntegration = await this.userRepository.getProjectIntegrationById(node.projectIntegrationId);
            const integrationValues = this.transformFormJSONValues(projectIntegration.formJSONValues);
            const providerName = this.getProvider(projectIntegration.masterIntegrationName);
            integrationValues.region = region;
            if (providerName === 'gce') {
                integrationValues.region = zone;
            }
            const saasEnvironment = configuration_1.ConfigFactory.config.getCustom('runtime.saasEnvironment');
            const customerName = configuration_1.ConfigFactory.config.getCustom('runtime.saasCustomerName');
            const isSass = configuration_1.ConfigFactory.config.isSaaS;
            const instanceVariables = new cloud_1.InstanceVariables(isSass, customerName, saasEnvironment);
            if (node.providerMetadataPropertyBag?.subscriptionId) {
                integrationValues.subscriptionId = node.providerMetadataPropertyBag.subscriptionId;
            }
            return cloud_1.CloudFactory.getProvider(providerName, integrationValues, instanceVariables);
        }
        catch (error) {
            throw new Error(`Failed to getNodeProvider due to errors ${(0, node_util_1.inspect)(error)}`);
        }
    }
    async getProviderRequestTransformer(providerType) {
        const runMode = configuration_1.ConfigFactory.config.runMode;
        const nodePollerIntervalMS = configuration_1.ConfigFactory.config.nodePollerIntervalMS;
        const customCertsEnabled = configuration_1.ConfigFactory.config.customCertsEnabled;
        const noVerifySsl = configuration_1.ConfigFactory.config.noVerifySsl;
        let externalApiUrl = await this.systemServiceRepository.getExternalURL();
        if (externalApiUrl.trim() == '') {
            externalApiUrl = configuration_1.ConfigFactory.config.externalApiUrl;
        }
        const customCertsData = this.getCustomCertData();
        switch (providerType.constructor) {
            case cloud_1.AwsProvider: {
                return new AwsRequestTransformer_1.AwsRequestTransformer(runMode, externalApiUrl, nodePollerIntervalMS, noVerifySsl, customCertsEnabled, customCertsData);
            }
            case cloud_1.GcloudProvider: {
                return new GcloudRequestTransformer_1.GcloudRequestTransformer(runMode, externalApiUrl, nodePollerIntervalMS, noVerifySsl, customCertsEnabled, customCertsData);
            }
            case cloud_1.KubernetesProvider: {
                return new K8sRequestTransformer_1.K8sRequestTransformer(runMode, externalApiUrl, nodePollerIntervalMS, customCertsEnabled, customCertsData);
            }
            case cloud_1.AzureProvider: {
                return new AzureRequestTransformer_1.AzureRequestTransformer(runMode, externalApiUrl, nodePollerIntervalMS, noVerifySsl, customCertsEnabled, customCertsData);
            }
            default: {
                throw new Error('No valid cloud provider found');
            }
        }
    }
    transformFormJSONValues(formValues) {
        const formKeyValues = {};
        for (const values of formValues) {
            formKeyValues[values.label] = values.value;
        }
        return formKeyValues;
    }
    getCustomCertData() {
        const certInitialzer = microservice_1.applicationContext.get(microservice_1.CERTIFICATE_INITIALIZER);
        return certInitialzer?.customCertsData;
    }
    getProvider(masterIntegrationName) {
        const providerName = providerType[masterIntegrationName];
        if (!providerName) {
            throw new Error(`Invalid provider name: ${providerName}`);
        }
        return providerName;
    }
}
exports.CloudServiceClient = CloudServiceClient;
//# sourceMappingURL=CloudServiceClient.js.map