"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.K8sRequestTransformer = void 0;
const cloud_1 = require("@jfrog-pipelines/cloud");
const configuration_1 = require("@jfrog-pipelines/configuration");
const data_access_layer_1 = require("@jfrog-pipelines/data-access-layer");
const userService_1 = require("../../../driven/acl/userService");
const BuildPlaneMessageDao_1 = require("../../../driven/db/BuildPlaneMessageDao");
const DefaultNamespace = `default`;
const rootlessDockerDataDirectory = '/home/rootless/.local/share/docker';
const jfrogDirectory = '/var/opt/jfrog/pipelines';
const requestKickDirectory = '/var/opt/jfrog/pipelines/reqKick';
const customCertsDirectory = '/var/opt/jfrog/pipelines/var/etc/security/keys/trusted';
const defaultK8sMemory = 5000;
const reqkickResourceMultiplier = 0.25;
const defaultK8sCpu = 1.5;
const dindResourceMultiplier = 0.75;
class K8sRequestTransformer {
    constructor(runMode, externalApiUrl, nodePollerIntervalMS, isCustomCertsEnabled, customCertsData) {
        this.runMode = runMode;
        this.externalApiUrl = externalApiUrl;
        this.nodePollerIntervalMS = nodePollerIntervalMS;
        this.isCustomCertsEnabled = isCustomCertsEnabled;
        this.customCertsData = customCertsData;
        this.userRepository = new userService_1.UserRepository();
        this.buildPlaneMessageDao = new BuildPlaneMessageDao_1.BuildPlaneMessageDao();
    }
    async getNodeCreateRequest(nodePool, node) {
        const project = await this.userRepository.getProjectById(nodePool.projectId);
        const providerMetadataPropertyBag = nodePool.providerMetadataPropertyBag;
        const imageId = providerMetadataPropertyBag.imageId;
        const namespace = providerMetadataPropertyBag.namespace ?? DefaultNamespace;
        const storageClass = providerMetadataPropertyBag.storageClass;
        const token = node.systemPropertyBag.token;
        const nodeName = node.friendlyName;
        const environments = await this.getEnviornmentVariables(nodePool, node);
        let requiredMemeory = configuration_1.ConfigFactory.config.getCustom('runtime.defaultK8sMemory') ?? defaultK8sMemory;
        requiredMemeory = providerMetadataPropertyBag?.memory ?? requiredMemeory;
        let requiredCpu = configuration_1.ConfigFactory.config.getCustom('runtime.defaultK8sCPU') ?? defaultK8sCpu;
        requiredCpu = providerMetadataPropertyBag?.cpu ?? requiredCpu;
        const k8sConfig = {
            namespace: namespace,
            pvcName: `${nodeName}-pvc`,
            secretName: `${nodeName}-secret`,
            storageClassName: storageClass,
            secretData: {
                token: Buffer.from(token).toString('base64')
            },
            customCertEnabled: this.isCustomCertsEnabled,
            Envs: environments,
            labels: providerMetadataPropertyBag.labels,
            reqkickImage: `${this.getRegistryUrl()}/jfrog/pipelines-reqkick:${node.intendedBuildPlaneVersion}`,
            dindImage: `${this.getDindRegistryUrl()}/jfrog/pipelines-dind:${node.intendedBuildPlaneVersion}`,
            reqkickCPU: `${requiredCpu * reqkickResourceMultiplier}`,
            reqkickMemory: `${requiredMemeory * reqkickResourceMultiplier}Mi`,
            dindMemory: `${requiredMemeory * dindResourceMultiplier}Mi`,
            dindCpu: `${requiredCpu * dindResourceMultiplier}`,
            poolType: node.runtime ? cloud_1.PoolKind.job : cloud_1.PoolKind.deployment,
            imagePullPolicy: this.runMode === 'devmode' ? 'Always' : 'IfNotPresent',
            initContainerEnv: [
                {
                    name: 'PIPELINES_NODE_API_TOKEN',
                    valueFrom: {
                        secretKeyRef: {
                            name: `${node.friendlyName}-secret`,
                            key: 'token'
                        }
                    }
                }
            ],
            insecureDockerRegistries: nodePool.initPropertyBag?.insecureDockerRegistries ?? [],
            initContainerCommand: this.getInitContainerCommand(node.id, nodePool.initPropertyBag?.insecureDockerRegistries),
            runOnePodInOneNode: configuration_1.ConfigFactory.config.runOnePodInOneNode
        };
        if (this.customCertsData) {
            k8sConfig.customCertData = { 'ca.crt': this.customCertsData };
        }
        const imageRegistrySecret = configuration_1.ConfigFactory.config.getCustom('core.imageRegistrySecret');
        if (imageRegistrySecret) {
            k8sConfig.imagePullSecret = [{ name: imageRegistrySecret }];
        }
        const instanceRequest = new cloud_1.CreateInstanceRequest(node.id, nodePool.providerMetadataPropertyBag.instanceSize, nodePool.operatingSystem, nodePool.id, nodePool.projectId, nodePool.projectIntegrationId, node.friendlyName, project.name, nodePool.name, nodePool.architecture, '', imageId, nodePool.providerMetadataPropertyBag.keyPairName, nodePool.providerMetadataPropertyBag.customTags, '', nodePool.diskSizeinGB, nodePool.providerMetadataPropertyBag.resourceGroupName, nodePool.providerMetadataPropertyBag.vnetName, undefined, undefined, nodePool.providerMetadataPropertyBag.publicSshKey, nodePool.providerMetadataPropertyBag.subnetId, nodePool.providerMetadataPropertyBag.securityGroupId, undefined, undefined, undefined, undefined, k8sConfig);
        return instanceRequest;
    }
    getNodeDeleteRequest(node) {
        const namespace = node.providerMetadataPropertyBag.namespace ?? DefaultNamespace;
        return new cloud_1.TerminateInstanceRequest([node.instanceId], undefined, undefined, undefined, undefined, namespace);
    }
    getNodeTransitionRequest(node) {
        const namespace = node.providerMetadataPropertyBag.namespace ?? DefaultNamespace;
        return new cloud_1.InstanceStateRequest(node.instanceId, undefined, namespace);
    }
    async getEnviornmentVariables(nodePool, node) {
        const urlConfig = await this.userRepository.getUnifiedUiAndWwwUrl();
        const environment = [
            {
                name: 'DOCKER_HOST',
                value: 'tcp://localhost:2376'
            },
            {
                name: 'DOCKER_CERT_PATH',
                value: '/certs/client'
            },
            {
                name: 'DOCKER_TLS_VERIFY',
                value: '1'
            },
            {
                name: 'NODE_ID',
                value: JSON.stringify(node.id)
            },
            {
                name: 'RUN_MODE',
                value: this.runMode
            },
            {
                name: 'PROJECT_ID',
                value: JSON.stringify(node.projectId)
            },
            {
                name: 'PIPELINES_NODE_ARCHITECTURE',
                value: nodePool.architecture
            },
            {
                name: 'PIPELINES_NODE_OPERATING_SYSTEM',
                value: nodePool.operatingSystem
            },
            {
                name: 'BASE_DIR',
                value: jfrogDirectory
            },
            {
                name: 'REQKICK_DIR',
                value: `${jfrogDirectory}/reqKick`
            },
            {
                name: 'REQEXEC_BIN_PATH',
                value: `${jfrogDirectory}/reqExec/dist/main/main`
            },
            {
                name: 'REQEXEC_DIR',
                value: `${jfrogDirectory}/reqExec`
            },
            {
                name: 'EXECTEMPLATES_DIR',
                value: `${requestKickDirectory}/node_modules/@jfrog-pipelines/core/execTemplates`
            },
            {
                name: 'EXTENSIONS_DIR',
                value: `${jfrogDirectory}/extensions`
            },
            {
                name: 'REPORTS_DIR',
                value: `${jfrogDirectory}/reports`
            },
            {
                name: 'BUILD_PLANE_VERSION',
                value: node.intendedBuildPlaneVersion
            },
            {
                name: 'PIPELINES_CORE_SERVICES_API_EXTERNALURL',
                value: this.externalApiUrl
            },
            {
                name: 'PIPELINES_SHARED_ARTIFACTORY_BASEURLUI',
                value: urlConfig.rtUiUrl
            },
            {
                name: 'PIPELINES_NODE_API_TOKEN',
                valueFrom: {
                    secretKeyRef: {
                        name: `${node.friendlyName}-secret`,
                        key: 'token'
                    }
                }
            },
            {
                name: 'IS_K8S_BUILD_PLANE',
                value: 'true'
            },
            {
                name: 'REQKICK_OS',
                value: 'UBI_8'
            },
            {
                name: 'NODE_POLLER_INTERVAL_MS',
                value: JSON.stringify(this.nodePollerIntervalMS)
            },
            {
                name: 'CUSTOM_CERTS_ENABLED',
                value: JSON.stringify(this.isCustomCertsEnabled)
            }
        ];
        if (this.isCustomCertsEnabled) {
            environment.push({
                name: 'CUSTOM_CERTS_DIR',
                value: customCertsDirectory
            }, {
                name: 'NODE_EXTRA_CA_CERTS',
                value: `${customCertsDirectory}/ca.crt`
            });
        }
        if (node.runtime) {
            environment.push({
                name: 'IS_BETA_K8S_BUILD_PLANE',
                value: 'true'
            }, {
                name: 'RUNTIME',
                value: `${node.runtime}`
            });
            const builder = new data_access_layer_1.SequelizeQueryBuilder({});
            builder.where('nodeId', node.id);
            const buildPlaneMessage = await this.buildPlaneMessageDao.findOne(builder.build());
            if (buildPlaneMessage) {
                environment.push({
                    name: 'BUILD_PLANE_MESSAGE_ID',
                    value: `${buildPlaneMessage.id}`
                });
            }
        }
        return environment;
    }
    getInitContainerCommand(nodeId, insecureDockerRegistries) {
        let dockerConfigDaemonCommands = '';
        let dockerConfigCertsCommands = '';
        if (insecureDockerRegistries && insecureDockerRegistries.length > 0) {
            dockerConfigDaemonCommands =
                `echo "Copying docker deamon configuration for insecure registries"; ` +
                    `cp -vpf /jfrog-pipelines/tmp/daemon.json /jfrog-pipelines-target/daemon.json; ` +
                    `chown -R 1000:1000 /jfrog-pipelines-target; `;
        }
        if (this.isCustomCertsEnabled) {
            dockerConfigCertsCommands =
                `echo "Copying custom certificate"; ` +
                    `cp -vpfr ${customCertsDirectory}/ca.crt ` +
                    `/usr/share/pki/ca-trust-source/anchors/; ` +
                    `echo "Installing custom certificate"; ` +
                    `update-ca-trust extract; `;
        }
        return [
            'sh',
            '-c',
            `cp -vpfr /jfrog-init/* /var/opt/jfrog/pipelines; ` +
                `chown -R 1000:1000 ${rootlessDockerDataDirectory}; ` +
                `${dockerConfigDaemonCommands}` +
                `${dockerConfigCertsCommands}` +
                `echo "Setting node initializedAt timestamp"; ` +
                `curl -sL -X PUT -H "Authorization: Bearer $PIPELINES_NODE_API_TOKEN" -H "Content-Type: application/json" -d '{"lastInitializedAt": "'$(date -u +"%FT%TZ")'"}' ${this.externalApiUrl}/nodes/${nodeId} -w "%{http_code}";` +
                `echo "node  has been initialized"; `
        ];
    }
    getRegistryUrl() {
        let coreRegistryUrl = configuration_1.ConfigFactory.config.getCustom('core.registryUrl');
        if (configuration_1.ConfigFactory.config.isSaaS) {
            coreRegistryUrl = 'releases-docker.jfrog.io';
        }
        return coreRegistryUrl;
    }
    getDindRegistryUrl() {
        let coreRegistryUrl = configuration_1.ConfigFactory.config.getCustom('core.registryUrl');
        if (configuration_1.ConfigFactory.config.isSaaS) {
            coreRegistryUrl = 'releases-docker.jfrog.io';
        }
        return coreRegistryUrl;
    }
}
exports.K8sRequestTransformer = K8sRequestTransformer;
//# sourceMappingURL=K8sRequestTransformer.js.map