"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodePool = exports.CloudProvider = void 0;
const node_assert_1 = require("node:assert");
const utility_1 = require("@jfrog-pipelines/utility");
const metadata_1 = require("@jfrog-pipelines/metadata");
const configuration_1 = require("@jfrog-pipelines/configuration");
const utils_1 = require("../../utils");
const acl_1 = require("../../driven/acl");
const MachineImageService_1 = require("../machine-image/MachineImageService");
const vo_1 = require("./vo");
var CloudProvider;
(function (CloudProvider) {
    CloudProvider["AWS"] = "AWS";
    CloudProvider["GCE"] = "GCE";
    CloudProvider["AZURE"] = "AZURE";
    CloudProvider["K8S"] = "K8S";
})(CloudProvider = exports.CloudProvider || (exports.CloudProvider = {}));
const providerType = {
    amazonKeys: CloudProvider.AWS,
    gcloudKey: CloudProvider.GCE,
    azureKeys: CloudProvider.AZURE,
    kubernetesConfig: CloudProvider.K8S
};
const EIGHT = 8;
const nodeAccessorID = '1';
const DEFAULT_SSH_USER = 'root';
class NodePool {
    constructor({ id, name, projectId, isCachedEnabled, nodeIdleIntervalInMins, cacheSettings, cloudProviderName, maxNodeCapacity, isOnDemand, projectIntegrationResponse, buildPlaneImageId, operatingSystem, architecture, initPropertyBag, diskSizeinGB, providerMetadataPropertyBag, initAsNonRoot }) {
        this.id = new utils_1.GenericVo(id);
        this.name = name;
        this.projectId = new utils_1.GenericVo(projectId);
        this.isCachedEnabled = isCachedEnabled;
        this.nodeIdleIntervalInMins = nodeIdleIntervalInMins;
        this.cacheSettings = cacheSettings;
        this.cloudProviderName = cloudProviderName;
        this.maxNodeCapacity = maxNodeCapacity;
        this.isOnDemand = isOnDemand;
        this.projectIntegrationResponse = projectIntegrationResponse;
        this.buildPlaneImageId = buildPlaneImageId;
        this.operatingSystem = new utils_1.GenericVo(operatingSystem);
        this.architecture = new utils_1.GenericVo(architecture);
        this.initPropertyBag = initPropertyBag;
        this.diskSizeinGB = new utils_1.GenericVo(diskSizeinGB);
        this.providerMetadataPropertyBag = providerMetadataPropertyBag;
        this.initAsNonRoot = initAsNonRoot;
        this.buildPlaneImageService = new MachineImageService_1.MachineImageService();
        this.userRepository = new acl_1.UserRepository();
        this.properties = configuration_1.ConfigFactory.config;
    }
    static async create(nodePool) {
        const cacheSettings = vo_1.CacheSettingsVo.create(NodePool.generateCacheSettingOptions(nodePool));
        const userRepository = new acl_1.UserRepository();
        const initPropertyBag = NodePool.generateInitPropertyBag(nodePool.initPropertyBag);
        const projectIntegrationResponse = await userRepository.getProjectIntegrationById(nodePool.projectIntegrationId);
        const cloudProviderName = NodePool.getProviderName(projectIntegrationResponse.masterIntegrationName);
        return new NodePool({
            id: `${nodePool.id}`,
            name: nodePool.name,
            projectId: `${nodePool.projectId}`,
            isCachedEnabled: nodePool.isCacheEnabled,
            nodeIdleIntervalInMins: nodePool.nodeIdleIntervalInMins,
            cacheSettings: cacheSettings,
            cloudProviderName: cloudProviderName,
            maxNodeCapacity: nodePool.numberOfNodes,
            isOnDemand: nodePool.isOnDemand,
            projectIntegrationResponse: projectIntegrationResponse,
            buildPlaneImageId: nodePool.buildPlaneImageId,
            operatingSystem: nodePool.operatingSystem,
            architecture: nodePool.architecture,
            initPropertyBag: initPropertyBag,
            diskSizeinGB: nodePool.diskSizeinGB ?? 0,
            providerMetadataPropertyBag: nodePool.providerMetadataPropertyBag,
            initAsNonRoot: nodePool.initAsNonRoot
        });
    }
    getId() {
        return this.id;
    }
    getProjectId() {
        return this.projectId;
    }
    async getProject() {
        const projectId = Number.parseInt(this.getProjectId().getValue(), 10);
        return this.userRepository.getProjectById(projectId);
    }
    getName() {
        return this.name;
    }
    getIsCachedEnabled() {
        return this.isCachedEnabled;
    }
    getNodeIdleIntervalInMins() {
        return this.nodeIdleIntervalInMins;
    }
    getCacheSettings() {
        return this.cacheSettings;
    }
    getCloudProviderName() {
        return this.cloudProviderName;
    }
    getMaxNodeCapacity() {
        return this.maxNodeCapacity;
    }
    isStatic() {
        return !this.isOnDemand;
    }
    getDiskSizeinGB() {
        return this.diskSizeinGB;
    }
    getOperatingSystem() {
        return this.operatingSystem;
    }
    getArchitecture() {
        return this.architecture;
    }
    getProjectIntegrationId() {
        return this.projectIntegrationResponse.id;
    }
    getProjectIntegrationResponse() {
        return this.projectIntegrationResponse;
    }
    getInitPropertyBag() {
        return this.initPropertyBag;
    }
    getInitAsNonRoot() {
        return this.initAsNonRoot;
    }
    async getNodeCreationAttributes(runtime) {
        const uid = (0, utility_1.uuidv4)();
        const providerMetadataPropertyBag = this.getProviderMetadataPropertyBag();
        const nodePoolProviderMetadataPropertyBag = { ...providerMetadataPropertyBag };
        const buildPlaneImageAttributes = await this.getBuildPlaneImage();
        const nodeRequestBody = {
            projectId: this.getProjectId().getValue(),
            nodePoolId: this.getId().getValue(),
            friendlyName: `node-${uid.slice(Math.max(0, uid.length - EIGHT))}`,
            isOnDemand: true,
            statusCode: metadata_1.WAITING,
            statusLastUpdatedAt: new Date(),
            createdBy: nodeAccessorID,
            updatedBy: nodeAccessorID,
            isAutoInitialized: false,
            stopRequested: false,
            intendedBuildPlaneVersion: this.getBuildPlaneVersion(),
            projectIntegrationId: this.getProjectIntegrationId(),
            sshPort: this.getSshPort(),
            sshUser: this.getUser(),
            systemPropertyBag: {},
            initPropertyBag: this.getInitPropertyBag(),
            providerMetadataPropertyBag: {
                instanceSize: nodePoolProviderMetadataPropertyBag.instanceSize,
                operatingSystem: this.getOperatingSystem().getValue(),
                uniqueToken: (0, utility_1.uuidv4)(),
                diskSizeinGB: this.getDiskSizeinGB().getValue(),
                imageMetadata: buildPlaneImageAttributes.metadataPropertyBag,
                imageId: buildPlaneImageAttributes.imageId,
                region: buildPlaneImageAttributes.region,
                zone: nodePoolProviderMetadataPropertyBag.zone,
                namespace: nodePoolProviderMetadataPropertyBag.namespace,
                storageClass: nodePoolProviderMetadataPropertyBag.storageClass,
                cpu: nodePoolProviderMetadataPropertyBag.cpu,
                memory: nodePoolProviderMetadataPropertyBag.memory,
                labels: nodePoolProviderMetadataPropertyBag.labels,
                subscriptionId: nodePoolProviderMetadataPropertyBag.subscriptionId,
                publicSshKey: nodePoolProviderMetadataPropertyBag.publicSshKey,
                subnetName: nodePoolProviderMetadataPropertyBag.subnetName,
                securityGroupName: nodePoolProviderMetadataPropertyBag.securityGroupName,
                resourceGroupName: nodePoolProviderMetadataPropertyBag.resourceGroupName,
                vnetName: nodePoolProviderMetadataPropertyBag.vnetName,
                adminUsername: nodePoolProviderMetadataPropertyBag.adminUsername,
                adminPassword: nodePoolProviderMetadataPropertyBag.adminPassword
            }
        };
        if (runtime) {
            nodeRequestBody.runtime = runtime;
        }
        return nodeRequestBody;
    }
    getProviderMetadataPropertyBag() {
        return this.providerMetadataPropertyBag;
    }
    getBuildPlaneVersion() {
        return this.properties.buildPlaneVersion ?? '';
    }
    getUser() {
        return (this.providerMetadataPropertyBag.sshUser ?? DEFAULT_SSH_USER);
    }
    getSshPort() {
        return (this.providerMetadataPropertyBag.sshPort ?? 0);
    }
    static getProviderName(masterIntegrationName) {
        const providerName = providerType[masterIntegrationName];
        if (!providerName) {
            throw new Error(`Invalid provider for masterIntegrationName ${masterIntegrationName}`);
        }
        return providerName;
    }
    async getBuildPlaneImage() {
        if (!this.buildPlaneImageId) {
            throw new node_assert_1.AssertionError({ message: 'BuildPlaneMessageId does not exist' });
        }
        const buildPlaneImage = await this.buildPlaneImageService.getBuildPlaneImageById(this.buildPlaneImageId);
        if (buildPlaneImage === null) {
            throw new Error(`BuildPlaneImage does not exist with id: ${this.buildPlaneImageId}`);
        }
        return buildPlaneImage;
    }
    static generateInitPropertyBag(initPropertyBag) {
        if (!initPropertyBag) {
            return {};
        }
        const propertyBag = initPropertyBag;
        if (!initPropertyBag.insecureDockerRegistries) {
            propertyBag.insecureDockerRegistries = [];
        }
        return propertyBag;
    }
    static generateCacheSettingOptions(nodePool) {
        const nodeStopSetting = nodePool.cacheSettingsPropertyBag
            .nodeStopSetting;
        if (nodeStopSetting === vo_1.CacheSettingType.nodeStopDayOfWeek) {
            return {
                settingType: vo_1.CacheSettingType.nodeStopDayOfWeek,
                config: nodePool.cacheSettingsPropertyBag.nodeStopDayOfWeek
            };
        }
        return {
            settingType: vo_1.CacheSettingType.nodeStopIntervalDays,
            config: nodePool.cacheSettingsPropertyBag.nodeStopIntervalDays
        };
    }
}
exports.NodePool = NodePool;
//# sourceMappingURL=NodePool.js.map