"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CreateStaticNodeHandler = void 0;
const node_path_1 = __importDefault(require("node:path"));
const node_util_1 = require("node:util");
const logger_1 = require("@jfrog-pipelines/logger");
const metadata_1 = require("@jfrog-pipelines/metadata");
const configuration_1 = require("@jfrog-pipelines/configuration");
const NodeDao_1 = require("../../driven/db/NodeDao");
const node_console_1 = require("../../domain/node-console");
const UserRepository_1 = require("../../driven/acl/userService/UserRepository");
const SystemServiceRepository_1 = require("../../driven/acl/systemService/SystemServiceRepository");
const SshExecutor_1 = require("../../domain/node/nexec/SshExecutor");
class CreateStaticNodeHandler {
    constructor(command) {
        this.command = command;
        this.logger = logger_1.LoggerFactory.getLogger('Nexec');
        this.tmpDir = '/tmp';
        this.nodeInitScriptPath = node_path_1.default.join(this.tmpDir, 'nodeInit.sh');
        this.DEFAULT_PORT = 22;
        this.nodeDao = new NodeDao_1.NodeDao();
        this.nodeLedger = new node_console_1.NodeConsoleLedgerService();
        this.userRepository = new UserRepository_1.UserRepository();
        this.systemServiceRepository = new SystemServiceRepository_1.SystemServiceRepository();
    }
    async handle() {
        const nodeId = this.command.getNodeId();
        if (!nodeId) {
            throw new Error(`node id cannot be empty`);
        }
        const node = await this.getNodeById(nodeId);
        try {
            const updatedNode = await this.updateNodeStatus(node, metadata_1.PROCESSING);
            const project = await this.getProjectById(updatedNode.projectId);
            const sshExecutor = await this.createSshConnection(node, project);
            await this.downloadAndExecuteInitScriptOnNode(sshExecutor);
            await this.nodeLedger.push(`Successfully initialized node with id ${node.id}`, node.id);
            this.logger.info(`Successfully initialized node with id ${nodeId}`);
        }
        catch (error) {
            await this.updateNodeStatus(node, metadata_1.FAILED);
            const errorMessage = (0, node_util_1.inspect)(error);
            await this.nodeLedger.push(`Node failed: ${errorMessage}`, node.id);
            throw new Error(`node failed due to error ${errorMessage}`);
        }
    }
    async getNodeById(nodeId) {
        const node = await this.nodeDao.findById(nodeId);
        if (!node) {
            await this.nodeLedger.push(`Node not found for id ${nodeId}`, nodeId);
            throw new Error(`Node not found for id ${nodeId}`);
        }
        return node;
    }
    async updateNodeStatus(node, statusCode) {
        const updateQuery = {
            statusCode: statusCode
        };
        const updatedNode = await this.nodeDao.updateByInstance(node, updateQuery);
        await this.nodeLedger.push('Updated node to processing', node.id);
        return updatedNode;
    }
    async getProjectById(projectId) {
        return this.userRepository.getProjectById(projectId);
    }
    async createSshConnection(node, project) {
        const sshExecutor = new SshExecutor_1.SshExecutor(this.getConnectionSettings(node, project), node.id);
        await sshExecutor.connect(this.nodeLedger.push.bind(this.nodeLedger));
        return sshExecutor;
    }
    async downloadAndExecuteInitScriptOnNode(sshExecutor) {
        let externalUrl = await this.systemServiceRepository.getExternalURL();
        if (externalUrl == '') {
            externalUrl = configuration_1.ConfigFactory.config.externalApiUrl;
        }
        const apiToken = configuration_1.ConfigFactory.config.getCustom('core.installerAuthToken');
        if (!apiToken) {
            throw new Error('No valid api token found');
        }
        const label = `Downloading node init script`;
        const nodeScriptDownloadUrl = `${externalUrl}/nodes/${sshExecutor.nodeId}/initScript`;
        const scriptPathOnNode = this.nodeInitScriptPath;
        const executionCommand = this.getExecutionCommand(apiToken, nodeScriptDownloadUrl, scriptPathOnNode);
        await sshExecutor.exec(executionCommand, label, this.nodeLedger.push.bind(this.nodeLedger));
        sshExecutor.end();
        return;
    }
    getExecutionCommand(apiToken, nodeScriptDownloadUrl, scriptPathOnNode) {
        return `
    sudo -E mkdir -p ${this.tmpDir};
    sudo -E rm ${this.nodeInitScriptPath} &> /dev/null;
    sudo -E curl -H "Authorization: apiToken ${apiToken}" ${nodeScriptDownloadUrl} --output ${scriptPathOnNode};
    sudo -E chmod +x ${scriptPathOnNode}; 
    sudo -E ${this.nodeInitScriptPath}; 
    sudo -E rm ${this.nodeInitScriptPath} &> /dev/null;`;
    }
    getConnectionSettings(node, project) {
        const connectionConfig = {
            host: node.IPAddress,
            port: node.sshPort || this.DEFAULT_PORT,
            username: node.sshUser,
            privateKey: project.systemPropertyBag.sshPrivateKey
        };
        return connectionConfig;
    }
}
exports.CreateStaticNodeHandler = CreateStaticNodeHandler;
//# sourceMappingURL=CreateStaticNodeHandler.js.map