"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const metadata_1 = require("@jfrog-pipelines/metadata");
const StatsDomainService_1 = require("../../../domain/node-stats/StatsDomainService");
const NodeStatsCheckHandler_1 = require("../NodeStatsCheckHandler");
const NodeStatsCheckCommand_1 = require("../../../driving/command/NodeStatsCheckCommand");
const acl_1 = require("../../../driven/acl");
const NodeDao_1 = require("../../../driven/db/NodeDao");
const NodeConsoleDao_1 = require("../../../driven/db/NodeConsoleDao");
jest.mock('../../../domain/node/NodeDomainService');
jest.mock('@jfrog-pipelines/internal-clients');
jest.mock('../../../driven/db/NodeConsoleDao');
jest.mock('@jfrog-pipelines/redis', () => {
    const actualRedis = jest.requireActual('@jfrog-pipelines/redis');
    return {
        ...actualRedis,
        RedisClientFactory: {
            client: {}
        }
    };
});
jest.mock('@jfrog-pipelines/configuration', () => {
    return {
        ConfigFactory: { config: { apiToken: 'apiToken', maxNodeCheckInDelayInMinutes: 15, jobConsoleBatchSize: 10 } }
    };
});
jest.mock('@jfrog-pipelines/logger', () => {
    const actualTracker = jest.requireActual('@jfrog-pipelines/logger');
    return {
        ...actualTracker,
        LoggerFactory: {
            getLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            }),
            configure: jest.fn()
        }
    };
});
describe('Node Stats Check Handler', () => {
    it('should return as no nodes were found in desired states', async () => {
        const getNodesByStatusQuery = {
            order: [['id', 'DESC NULLS LAST']],
            where: { deletedAt: null, statusCode: { $in: [metadata_1.WAITING, metadata_1.SUCCESS, metadata_1.PROCESSING] }, stopRequested: 'false' }
        };
        const getNodesByStatusSpy = jest.spyOn(NodeDao_1.NodeDao.prototype, 'findAll').mockResolvedValue([]);
        const isNodeActiveSpy = jest.spyOn(StatsDomainService_1.StatsDomainService.prototype, 'getLatestNodeStatForNode');
        const command = new NodeStatsCheckCommand_1.NodeStatsCheckCommand();
        await new NodeStatsCheckHandler_1.NodeStatsCheckHandler(command).handle();
        expect(getNodesByStatusSpy).toHaveBeenCalledTimes(1);
        expect(getNodesByStatusSpy).toHaveBeenCalledWith(getNodesByStatusQuery);
        expect(isNodeActiveSpy).toHaveBeenCalledTimes(0);
    });
    it('should return as node fetched is active and receiving nodeStats', async () => {
        const getNodesByStatusQuery = {
            order: [['id', 'DESC NULLS LAST']],
            where: { deletedAt: null, statusCode: { $in: [metadata_1.WAITING, metadata_1.SUCCESS, metadata_1.PROCESSING] }, stopRequested: 'false' }
        };
        const created16minBefore = 960000;
        const nodeOutput = [
            {
                id: 1,
                projectId: 1,
                stepId: 1,
                createdAt: new Date(Date.now() - created16minBefore).toISOString()
            }
        ];
        const reportedBefore6min = 600000;
        const latestNodeStatOutput = {
            id: 1,
            nodeId: 1,
            diskUsageInPercentage: 20.3,
            projectId: 1,
            activeContainersCount: 20,
            reportedAt: Date.now() - reportedBefore6min
        };
        const getNodesByStatusSpy = jest.spyOn(NodeDao_1.NodeDao.prototype, 'findAll').mockResolvedValueOnce(nodeOutput);
        const isNodeActiveSpy = jest
            .spyOn(StatsDomainService_1.StatsDomainService.prototype, 'getLatestNodeStatForNode')
            .mockResolvedValueOnce(latestNodeStatOutput);
        const updateNodeStatusSpy = jest.spyOn(NodeDao_1.NodeDao.prototype, 'update');
        const command = new NodeStatsCheckCommand_1.NodeStatsCheckCommand();
        await new NodeStatsCheckHandler_1.NodeStatsCheckHandler(command).handle();
        expect(getNodesByStatusSpy).toHaveBeenCalledTimes(1);
        expect(getNodesByStatusSpy).toHaveBeenCalledWith(getNodesByStatusQuery);
        expect(isNodeActiveSpy).toHaveBeenCalledTimes(1);
        expect(isNodeActiveSpy).toHaveBeenCalledWith(nodeOutput[0].id);
        expect(updateNodeStatusSpy).toHaveBeenCalledTimes(0);
    });
    it('should update node status to failure but then return since stepId is null', async () => {
        const updateNodeStatusQuery = {
            where: {
                id: 1
            }
        };
        const updateNodeStatusData = {
            statusCode: metadata_1.FAILED
        };
        const getNodesByStatusQuery = {
            order: [['id', 'DESC NULLS LAST']],
            where: { deletedAt: null, statusCode: { $in: [metadata_1.WAITING, metadata_1.SUCCESS, metadata_1.PROCESSING] }, stopRequested: 'false' }
        };
        const created16minBefore = 960000;
        const nodeOutput = [
            {
                id: 1,
                projectId: 1,
                stepId: null,
                createdAt: new Date(Date.now() - created16minBefore).toISOString()
            }
        ];
        const reportedBefore18min = 1080000;
        const latestNodeStatOutput = {
            id: 1,
            nodeId: 1,
            diskUsageInPercentage: 20.3,
            projectId: 1,
            activeContainersCount: 20,
            reportedAt: new Date(Date.now() - reportedBefore18min).toISOString()
        };
        const getNodesByStatusSpy = jest.spyOn(NodeDao_1.NodeDao.prototype, 'findAll').mockResolvedValueOnce(nodeOutput);
        const isNodeActiveSpy = jest
            .spyOn(StatsDomainService_1.StatsDomainService.prototype, 'getLatestNodeStatForNode')
            .mockResolvedValueOnce(latestNodeStatOutput);
        const updateNodeStatusSpy = jest.spyOn(NodeDao_1.NodeDao.prototype, 'update').mockImplementationOnce(jest.fn());
        const getStepById = jest.spyOn(acl_1.StepRepository.prototype, 'getStepById');
        const command = new NodeStatsCheckCommand_1.NodeStatsCheckCommand();
        await new NodeStatsCheckHandler_1.NodeStatsCheckHandler(command).handle();
        expect(getNodesByStatusSpy).toHaveBeenCalledTimes(1);
        expect(getNodesByStatusSpy).toHaveBeenCalledWith(getNodesByStatusQuery);
        expect(isNodeActiveSpy).toHaveBeenCalledTimes(1);
        expect(isNodeActiveSpy).toHaveBeenCalledWith(nodeOutput[0].id);
        expect(updateNodeStatusSpy).toHaveBeenCalledTimes(1);
        expect(updateNodeStatusSpy).toHaveBeenCalledWith(updateNodeStatusData, updateNodeStatusQuery);
        expect(getStepById).toHaveBeenCalledTimes(0);
    });
    it('should update node status to failure and timeout steps', async () => {
        const updateNodeStatusQuery = {
            where: {
                id: 1
            }
        };
        const updateNodeStatusData = {
            statusCode: metadata_1.FAILED
        };
        const getNodesByStatusQuery = {
            order: [['id', 'DESC NULLS LAST']],
            where: { deletedAt: null, statusCode: { $in: [metadata_1.WAITING, metadata_1.SUCCESS, metadata_1.PROCESSING] }, stopRequested: 'false' }
        };
        const created16minBefore = 960000;
        const nodeOutput = [
            {
                id: 1,
                projectId: 1,
                stepId: 1,
                createdAt: new Date(Date.now() - created16minBefore).toISOString()
            }
        ];
        const reportedBefore18min = 1080000;
        const latestNodeStatOutput = {
            id: 1,
            nodeId: 1,
            diskUsageInPercentage: 20.3,
            projectId: 1,
            activeContainersCount: 20,
            reportedAt: new Date(Date.now() - reportedBefore18min).toISOString()
        };
        const getStepByIdOutput = {
            id: 1,
            runId: 1,
            affinityGroup: 'group'
        };
        const getStepsQueuedToNodeOutput = [
            {
                id: 1,
                runId: 1,
                affinityGroup: 'group'
            },
            {
                id: 2,
                runId: 1,
                affinityGroup: 'group'
            }
        ];
        const getNodesByStatusSpy = jest.spyOn(NodeDao_1.NodeDao.prototype, 'findAll').mockResolvedValueOnce(nodeOutput);
        const isNodeActiveSpy = jest
            .spyOn(StatsDomainService_1.StatsDomainService.prototype, 'getLatestNodeStatForNode')
            .mockResolvedValueOnce(latestNodeStatOutput);
        const updateNodeStatusSpy = jest.spyOn(NodeDao_1.NodeDao.prototype, 'update').mockImplementationOnce(jest.fn());
        const getStepById = jest.spyOn(acl_1.StepRepository.prototype, 'getStepById').mockResolvedValueOnce(getStepByIdOutput);
        const getStepsQueuedToNodeSpy = jest
            .spyOn(acl_1.StepRepository.prototype, 'getStepsQueuedToTheNode')
            .mockResolvedValueOnce(getStepsQueuedToNodeOutput);
        const timeoutStepsSpy = jest.spyOn(acl_1.StepRepository.prototype, 'timeOutSteps').mockResolvedValueOnce();
        const nodeConsoleDaoSpy = jest.spyOn(NodeConsoleDao_1.NodeConsoleDao.prototype, 'create').mockImplementationOnce(jest.fn());
        const command = new NodeStatsCheckCommand_1.NodeStatsCheckCommand();
        await new NodeStatsCheckHandler_1.NodeStatsCheckHandler(command).handle();
        expect(getNodesByStatusSpy).toHaveBeenCalledTimes(1);
        expect(getNodesByStatusSpy).toHaveBeenCalledWith(getNodesByStatusQuery);
        expect(isNodeActiveSpy).toHaveBeenCalledTimes(1);
        expect(isNodeActiveSpy).toHaveBeenCalledWith(nodeOutput[0].id);
        expect(updateNodeStatusSpy).toHaveBeenCalledTimes(1);
        expect(updateNodeStatusSpy).toHaveBeenCalledWith(updateNodeStatusData, updateNodeStatusQuery);
        expect(getStepById).toHaveBeenCalledTimes(1);
        expect(getStepById).toHaveBeenCalledWith(nodeOutput[0].id);
        expect(getStepsQueuedToNodeSpy).toHaveBeenCalledTimes(1);
        expect(getStepsQueuedToNodeSpy).toHaveBeenCalledWith(getStepByIdOutput.affinityGroup, getStepByIdOutput.runId);
        expect(timeoutStepsSpy).toHaveBeenCalledTimes(1);
        expect(timeoutStepsSpy).toHaveBeenCalledWith(getStepsQueuedToNodeOutput);
        expect(nodeConsoleDaoSpy).toHaveBeenCalledTimes(1);
    });
});
//# sourceMappingURL=NodeStatsCheckHandler.test.js.map