"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProcessNodePoolHandler = exports.LOCK_TIMEOUT_IN_MS = void 0;
const node_util_1 = require("node:util");
const microservice_1 = require("@jfrog-pipelines/microservice");
const logger_1 = require("@jfrog-pipelines/logger");
const NodeCollector_1 = require("../../domain/node/NodeCollector");
const NodeProvisioner_1 = require("../../domain/node/NodeProvisioner");
const NodePoolService_1 = require("../../domain/nodepool/NodePoolService");
const FeatureAwareFactory_1 = require("../../shared/FeatureAwareFactory");
exports.LOCK_TIMEOUT_IN_MS = 60000;
class ProcessNodePoolHandler {
    constructor(command, serviceMetaData) {
        this.command = command;
        this.serviceMetaData = serviceMetaData;
        this.logger = logger_1.LoggerFactory.getLogger('nodePoolWorker');
        this.nodePoolService = new NodePoolService_1.NodePoolService();
    }
    async handle() {
        const poolID = this.command.getNodePoolId();
        const systemInstanceId = microservice_1.applicationContext.get(microservice_1.SYSTEM_SERVICE_INSTANCE_INITIALIZER)
            ?.systemServiceInstancesId;
        this.logger.trace(`Processing for node pool with id: ${poolID} with system service id: ${systemInstanceId}`);
        const lockName = [this.serviceMetaData.name, '-nodePools-', ...String(poolID)].join('');
        const lock = new microservice_1.Lock(systemInstanceId, lockName, exports.LOCK_TIMEOUT_IN_MS);
        try {
            const onDemandNodeStartTime = Date.now();
            const nodePool = await this.nodePoolService.getNodePool(poolID);
            if (!nodePool) {
                this.logger.warn(`NodePool id ${poolID} is not dynamic nodepool`);
                return;
            }
            this.logger.trace(`Successfully fetched node pool with id: ${poolID} and node pool name is ${nodePool.name}`);
            if (!nodePool.isOnDemand) {
                this.logger.info(`Node Pool is not on demand, wont continue with execution`);
                return;
            }
            const certInitializer = microservice_1.applicationContext.get(microservice_1.CERTIFICATE_INITIALIZER);
            const collector = new NodeCollector_1.NodeCollector();
            const provisioner = new NodeProvisioner_1.NodeProvisioner(certInitializer);
            const lockResult = await lock.obtainLock();
            if (!lockResult) {
                this.logger.info(`Failed to obtain lock for node pool id ${poolID}`);
                return;
            }
            this.logger.trace(`Successfully obtained lock for node pool ${nodePool.name} with lock name: ${lockName}`);
            const onDemandNodeEndTime = Date.now();
            const onDemandNodeTimeDiff = new Date(onDemandNodeEndTime - onDemandNodeStartTime);
            const requiredNodes = await this.getRequiredNodes(poolID);
            const onDemandNodeData = {
                domain: 'nodepool',
                eventType: 'onDemandNodes',
                durationMillis: onDemandNodeTimeDiff.valueOf(),
                count: requiredNodes,
                nodePoolName: nodePool.name
            };
            logger_1.LoggerFactory.logEvent(onDemandNodeData);
            this.logger.trace(`Required Node count is ${requiredNodes}`);
            if (nodePool.isBeta) {
                this.logger.trace('Beta node pool, will not de-provision node');
                await provisioner.provisionBeta(nodePool);
            }
            else {
                await provisioner.provision(nodePool, requiredNodes);
                await collector.deProvision(nodePool, requiredNodes);
                await this.collectOrphanNodes(collector);
            }
        }
        catch (error) {
            this.logger.error(`Failed to process node pool id ${poolID} with err ${(0, node_util_1.inspect)(error)}`);
        }
        finally {
            await lock.releaseLock();
            this.logger.trace(`Lock was released successfully with name: ${lockName}`);
        }
    }
    async getRequiredNodes(poolID) {
        const features = new FeatureAwareFactory_1.FeatureAwareFactory();
        return features.getRequiredNodes(poolID);
    }
    async collectOrphanNodes(collector) {
        this.logger.debug('Collecting orphan nodes started');
        const collectedNodeCount = await collector.collectJunkNodes();
        this.logger.debug(`${collectedNodeCount} orphan nodes were successfully collected`);
    }
}
exports.ProcessNodePoolHandler = ProcessNodePoolHandler;
//# sourceMappingURL=ProcessNodePoolHandler.js.map